#include<iostream>
#include<fstream>

using namespace std;



#include"defo.h"
#include"multiint.h"
#include"zdynsys.h"
#include"diffincl.h"
#include"zgfiles.h"
#include"kspolbds.h"


#define NEW_INTEXP
// introduced 3.VI.2008 for test of new method of computation of \int_0^te^{j(t-s)}ds
//  for small time steps it slows downs the computations

// constants used in the computation of the difference of solutions
// of differential inclusion x'\in f(x) + [-delta,delta]
// and                       y'=f(y)
const int Ldelta_lognorm=2;  // an approach based on logarithmic norm
const int Ldelta_componentwiseestm=3; // an approach based on componentwise estimates
// *********************************


// constants used to describe verious approaches to perform one step in the integration
// of differential inclusions
const int dfincl_C0simpleLo =1; // simple C0-method Lohner method
const int dfincl_C0thruC1Lo =2; // deterministic part is computed using C1-Lohner
// **********************



char *inclmethoddescr(char* name,int maxlen,int method)
// returns the name of Ldeta-method - see comments regarding constants
//  Ldelta_lognorm, Ldelta_componentwiseestm
//  maxlen - the size of the table name[]
{
  strcpy(name,"");
  if(method==Ldelta_lognorm){
    strncpy(name,"based on logarithmic norm",maxlen-1);
  }
  if(method==Ldelta_componentwiseestm){
    strncpy(name,"based on componentwise estimates",maxlen-1);
  }

  return name;
}

int ask_inclmethod()
// the choice between the method based on logarithmic norm and conponent-wise estimates
{
  const int buflen=60;
  char buffer[buflen];
  cout << " select inclusion computation method " << endl;
  cout << " 1 -  " << inclmethoddescr(buffer,buflen,Ldelta_lognorm) << endl;
  cout << " 2 -  "  << inclmethoddescr(buffer,buflen,Ldelta_componentwiseestm) << endl;
  int ch;
  int Ldelta_method;
  cin >> ch;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  if(ch==1) Ldelta_method=Ldelta_lognorm;
  else{
    // if(ch==2) Ldelta_method= Ldelta_componentwiseestm;
    Ldelta_method= Ldelta_componentwiseestm;
  }
  return  Ldelta_method;
}

void incl_method_chose(int *ldeltamethod,int *plognorm)
//  if *ldeltamethod==Ldelta_componentwiseestm, then <*plognorm> is ignored
{
  *ldeltamethod= ask_inclmethod();
  *plognorm=0;
  if(*ldeltamethod==Ldelta_lognorm){
    const int buflen=40;
    char buffer[buflen];
    cout << " select the norm " << endl;
    cout << NORM_SUM <<  " -  " << norm_name(buffer,buflen,NORM_SUM) << endl;
//    cout << NORM_EUCL << " -  " << norm_name(buffer,buflen,NORM_EUCL)<< endl;
    // for euclidean norm the function is not rigorous
    cout << NORM_MAX  << " -  " << norm_name(buffer,buflen,NORM_MAX) << endl;
    cin >> *plognorm;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }
}


char *incl_onestepmethod(char* name,int maxlen,int method)
// returns the name (a description) of way we perform one step of the integration
// of differential inclusions
//  maxlen - the size of the table name[]
{
  strcpy(name,"");
  if(method==dfincl_C0simpleLo){
    strncpy(name,"C^0-Lohner method",maxlen-1);
  }
  if(method==dfincl_C0thruC1Lo){
    strncpy(name,"C^1-Lohner method for deterministic part",maxlen-1);
  }

  return name;
}


void selinclonestepmethod(int *onestepmethod,int *orderdf)
// we select from the console the one step method for diff. incl. if necessary we also
//  ask for for the order of C^1-part
{
  const int buflen=80;
  char buffer[buflen];
  cout << "select one step method for differential inclusion: " << endl;
  cout << dfincl_C0simpleLo << " - " << incl_onestepmethod(buffer,buflen,dfincl_C0simpleLo) << endl;
  cout << dfincl_C0thruC1Lo << " - " << incl_onestepmethod(buffer,buflen,dfincl_C0thruC1Lo) << endl;
  cin >> *onestepmethod;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  if(*onestepmethod!=dfincl_C0thruC1Lo) // setting the default value
    *onestepmethod=dfincl_C0simpleLo;

  if(*onestepmethod==dfincl_C0thruC1Lo){
    cout << "order of the C^1-part " ;
    cin >> *orderdf;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }
}


InclOneStepParamType::InclOneStepParamType()
{}

InclOneStepParamType::InclOneStepParamType(int pmethod,int porder,int porderdf,
           int pdoC0intersection,int ponestepmethod)
{
  method=pmethod;
  order=porder;
  orderdf=porderdf;
  doC0intersection=pdoC0intersection;
  onestepmethod=ponestepmethod;
}

void  InclOneStepParamType::write(ostream &report) const
{
  report << "order=" << order << endl;
  if(doC0intersection){
    report << "using the intersection with the dissipative enclosure "  << endl;
  }
  const int buflen=80;
  char buffer[buflen];
  report <<  incl_onestepmethod(buffer,onestepmethod,buflen) << endl;
  if(onestepmethod==dfincl_C0thruC1Lo){
    report  << "C1 - order " << orderdf << endl;
  }
}


InclPoincareParType::InclPoincareParType(double ph,double ptmax,int pmethod,
           int porder,int porderdf,int pdoC0intersection,int ponestepmethod)
{
  h=ph;
  tmax=ptmax;
  onesteppar.method=pmethod;
  onesteppar.order=porder;
  onesteppar.orderdf=porderdf;
  onesteppar.doC0intersection=pdoC0intersection;
  onesteppar.onestepmethod=ponestepmethod;
}

InclPoincareParType::InclPoincareParType()
{
}

void InclPoincareParType::write(ostream &report) const
{
  report << "the time step h=" << h << endl;
  report << "tmax=" << tmax << endl;
  onesteppar.write(report);
}

void copyToParPoincare(InclPoincareParType &source,Par_Poincare &dest)
{
  dest.tmax=source.tmax;
  dest.h=source.h;
  dest.onesteppar.method=source.onesteppar.method;
  dest.onesteppar.order=source.onesteppar.order;
  dest.onesteppar.orderdf=source.onesteppar.orderdf;
  dest.onesteppar.doC0intersection=source.onesteppar.doC0intersection;
}



/// InclOde
void InclOde::incl_field(const interval x[],SetWithTail &yestm,interval f[])
{
  for(int i=1;i<=dim;i++)
    f[i]=incl_vfield(i,x,yestm);
}


void InclOde::set_yc(interval new_y_c[])
{
  for(int i=1;i<=dim;i++)
    y_c[i]=new_y_c[i];
  todouble(y_c,y_c_double,dim);
}
void InclOde::current_yc(interval y[])
{
  copyvector(y_c,y,dim);
}

void InclOde::set_ycenter(interval y[])
{
//  interval new_yc[MAX_DIM+1];
  interval *new_yc;
  new_yc=new interval[dim+1];
  for(int i=1;i<=dim;i++)
    new_yc[i]=mid(y[i]);
  set_yc(new_yc);
  delete [] new_yc;
}


void  InclOde::delta_field(const interval x[],SetWithTail & estmy,interval deltaf[])
 //   deltaf= f(x,estmy)-  f(x,y_c)   for x \in <x>
{
  for(int i=1;i<=dim;i++)
    deltaf[i]=delta_vf(i,x,estmy);
}






int InclOde::incl_Ldelta_lognorm(interval h,interval w1_encl[],interval w2_encl[],
      SetWithTail & wy,interval Ldelta[])
//  computes \Delta (Ldelta) - with symbols w1,w2,wy - as in my notes
//    \Delta - is the difference between the segment form ODE with y=y_c
//                and the inclusion uder consideration
//  w1_encl - is a rough enclosure for x1[0,h] - the solution of x'=f(x,y_c)
//  w2_encl - a roght enclosure for x2[[0,h] - the solution of x' \in f(x,[y])
//  wy - the rough enclosure for y-variables
{
  int ok=1;
//  interval delta[MAX_DIM+1];
  interval *delta;
  delta =new interval[dim+1];
  delta_field(w1_encl,wy,delta);

//  interval dfdx[MAX_DIM+1][MAX_DIM+1];
  intervalmatrix1 *dfdx;
  dfdx=new intervalmatrix1[dim+1];
  for(int i=1;i<=dim;i++)
    for(int j=1;j<=dim;j++)
       dfdx[i][j]=incl_dvfield(i,j,w2_encl,wy);

  interval norm_delta=Right(norm(delta,dim,Ldelta_norm));
  interval l=Right(logarithmic_norm(dfdx,dim,Ldelta_norm));


  interval coeff;
  if(l==0) coeff=Right(h);

  if(l !=0)  coeff=Right((exp(l*h)-1.0)/l);
  if(l<0){
    if(Right(coeff) > Right(h)){  // I imagine this can happen when l is very close
                        // to 0; but coeff = (exp(l*h) -1)/l is an increasing function
                        //  of l ; hence I'm comparing with what I get for l=0
      coeff=Right(h);
    }
  }
  if(l>0){
    // comparison with 1.01*l ; as above we want to treat the case of <l> very small
    interval ll=Right(1.01*l);
    interval tc=Right((exp(ll*h)-1)/ll);
    if(tc < coeff) coeff=tc;
  }
  for(int i=1;i<=dim;i++)
    Ldelta[i]=interval(-1,1)*norm_delta*coeff;


  delete [] dfdx;
  delete [] delta;
  return ok;
}

bool compute_exp_integral(interval t,interval a[][MAX_DIM+1],int pdim,interval
  minte[][MAX_DIM+1])
 // on output minte=\int_0^t e^{a(t-s)} ds
{
  double eps=1e-3;
  const int max_N=100;
  const int normfunc=NORM_MAX; // only NORM_MAX and NORM_SUM -can be used -
                // NORM_EUCL - is not rigorous


  int ok=1;

//    interval ta[MAX_DIM+1][MAX_DIM+1]; //{\tilde A}
  intervalmatrix1 *ta;  // \tilde{A}
  ta=new intervalmatrix1[pdim+1];
  for(int i=1;i<=pdim;i++)
    for(int j=1;j<=pdim;j++)
      ta[i][j]=t*a[i][j];

  interval norm_ta=Right(norm(ta,pdim,normfunc));


//  interval an[MAX_DIM+1][MAX_DIM+1];
  intervalmatrix1 *an;
  an=new intervalmatrix1[pdim+1];
  identity(an,pdim);
  int n=0;

  interval rem;
  copymatrix(an,minte,pdim);
  int finished=0;
  while((!finished) && (n < max_N) && ok){

    // we compute a_{n+1}
    ArrArr(an,ta,an,pdim);
    for(int i=1; i<=pdim;i++)
      for(int j=1;j<=pdim; j++)
        an[i][j] /= (n+2);
    n=n+1;

    for(int i=1;i<=pdim;i++)
      for(int j=1;j<=pdim;j++)
        minte[i][j]+=an[i][j];


    //  checking the  stopping condition
    interval n_an=norm(an,pdim,normfunc);
    interval q=norm_ta/(n+2);
    if(q < 1){
      rem=Right(n_an*q/(1-q));  // this is a remainder term
//      if(rem < eps*norm(minte,pdim,normfunc)) finished=1;
      if(rem < eps*norm_ta) finished=1;
    }
  }
  if(!finished) ok=0;


  if(ok){
    rem=Ihull(-rem,rem);
    for(int i=1;i<=pdim;i++)
      for(int j=1;j<=pdim;j++){
        minte[i][j]+=rem;
        minte[i][j]*=t;
      }
  }
  delete [] an;
  delete [] ta;
  return ok;
}



bool  smart_compute_exp_integral(interval t,interval A[][MAX_DIM+1],int pdim,interval
     G[][MAX_DIM+1], int level =0   )
 // on output G=G(t,A)\int_0^t e^{A(t-s)} ds
//    level - indicates how deep in the recurrence are we
// the computation is based on the following observations:
//  G(t,J)=[exp(J*t/2) + Id]G(t/2,J)
//  G(t,J)=t*G(1,tJ)
//
{
  interval bigNorm=3.0;

  bool retval=false;
  interval normA=t*Right(norm(A,pdim,NORM_MAX));
  if(normA > bigNorm){
    interval t_half=t/2;
    intervalmatrix1 *At2=new intervalmatrix1[pdim+1];
    intervalmatrix1 *expAt2=new intervalmatrix1[pdim+1];
    copymatrix(A,At2,pdim);
    for(int i=1;i<=pdim;i++)
      for(int j=1;j<=pdim;j++)
        At2[i][j]*=t_half;


    retval= smart_matrixexp(At2,pdim,expAt2);
    if(retval){
      for(int i=1;i<=pdim;i++)
        expAt2[i][i]+=interval(1.0);
      // now expAt2= exp(A*t/2) + Id

      intervalmatrix1 *Gt2=new intervalmatrix1[pdim+1];
      retval= smart_compute_exp_integral(t_half,A,pdim,Gt2, level+1 );

      if(retval){
        ArrArr(expAt2,Gt2,G,pdim);
      }else{
 cerr << "failed to compute  in <smart_compute_exp_integral> - level=" << level << endl;
      }

      delete [] *Gt2;
    } else{
      cerr << "failed to compute exp in <smart_compute_exp_integral>" << endl;
    }

    delete [] expAt2;
    delete [] At2;
  }else{
    retval=compute_exp_integral(t,A,pdim,G);
  }
  return retval;
}

void write_error_inclexpintegral(interval t,interval a[][MAX_DIM+1],int dim,
   interval minte[][MAX_DIM+1])
// report about the failure of the function <compute_exp_integral>
{
  ofstream report(zglerrorlogname,ios::app);
  entriesinerrorlog=1;
  report << "-------------------------" << endl;
  writeCompInfo(report);
  writedatetime(report);
  report << "failed to compute \\int_0^t e^(a(t-s))ds" << endl;

  report << "t=" << t << endl;
  report << "dim=" << dim << endl;
  report << "Matrix a:" << endl;
  saveArray(report,a,dim,1,(dim > 8)); // mid point only is written
  report << endl << endl;

  report << "Matrix a:" << endl;
  saveArray(report,a,dim,0,(dim > 8)); // mid point only is written
  report << endl << endl;

  report << "Matrix minte:" << endl;
  saveArray(report,minte,dim,1,(dim > 8));
  report << endl << endl;

}



int  InclOde::incl_Ldelta_componentwise( interval h,
        interval w1_encl[],interval w2_encl[],SetWithTail & wy,interval Ldelta[])
//  computes \Delta (Ldelta) - with symbols w1,w2,wy - as in my notes
//  using an approach based on componentwise estimates
//  w1_encl - is a rough enclosure for x1[0,h] - the solution of x'=f(x,y_c)
//  w2_encl - a rough enclosure for x2[0,h] - the solution of x' \in f(x,[y])
//  wy - the rough enclosure for y-variables
{

  interval delta[MAX_DIM+1];
  delta_field(w1_encl,wy,delta);

  interval c[MAX_DIM+1];
  for(int i=1;i<=dim;i++)
    c[i]=Right(abs(delta[i]));


  intervalmatrix1 *b;
  b=new intervalmatrix1[dim+1];
  for(int i=1;i<=dim;i++)
    for(int j=1;j<=dim;j++)
       b[i][j]=incl_dvfield(i,j,w2_encl,wy);


  for(int i=1;i<=dim;i++)
    for(int j=1;j<=dim;j++){
      if(i==j){
        b[i][j]= Right(b[i][j]);
      } else  b[i][j]= Right(abs(b[i][j]));
    }

//  interval minte[MAX_DIM+1][MAX_DIM+1];
  intervalmatrix1 *minte;
  minte=new intervalmatrix1[dim+1];
  #if defined(NEW_INTEXP)
  int ok=smart_compute_exp_integral(h,b,dim,minte); // on output minte=\int_0^h e^{b(h-s)} ds
  #else
  int ok=compute_exp_integral(h,b,dim,minte); // on output minte=\int_0^h e^{b(h-s)} ds
  #endif

  if(ok){
    interval d[MAX_DIM+1];
    multAV(minte,c,d,dim);

    for(int i=1;i<=dim;i++)
      Ldelta[i]=interval(-d[i].right_b(),d[i].right_b());
  }else{
    write_error_inclexpintegral(h,b,dim,minte);
    cerr << "unable to compute the integral \\int_0^h e^{b(h-s)} ds "<< endl;
/*    #if defined(KRAK) && defined(WIN95)
    waitBt();
    #endif */
  }
  delete [] minte;
  delete [] b;
  return ok;
}


int InclOde::incl_compute_Ldelta(interval h,
        interval w1_encl[],interval w2_encl[],SetWithTail & wy,interval Ldelta[])
// computes \Delta (Ldelta) - with symbols w1,w2,wy - as in my notes
//  w1_encl - is a rough enclosure for x1[0,h] - the solution of x'=f(x,y_c)
//  w2_encl - a roght enclosure for x2[[0,h] - the solution of x' \in f(x,[y])
//  wy - the rough enclosure for y-variables
{
  int ret=0;

  if(Ldelta_method==Ldelta_lognorm)  // an approach based on logarithmic norm
    ret=incl_Ldelta_lognorm(h,w1_encl,w2_encl,wy,Ldelta);

  if(Ldelta_method==Ldelta_componentwiseestm) // an approach on componentwise estimates
    ret=incl_Ldelta_componentwise(h,w1_encl,w2_encl,wy,Ldelta);

  return ret;
}



void InclOde::incl_fromsegdist(interval w1_encl[],interval w2_encl[],
       SetWithTail &wy, interval dist[],interval h)
  //  function computes a distance from segment joinnig end points
  //  of the "unperturbed ode" (i.e.  with fixed y_c)
  // useful in section region
{
  interval h0h=Ihull(interval(0),h);
  interval Ldelta[MAX_DIM+1];
  if(!incl_compute_Ldelta(h0h,w1_encl,w2_encl,wy,Ldelta)){
    cerr << "computation of LDelta failed" << endl;
      #if defined(KRAK) && defined(WIN95)
           waitBt();
       #endif
    exit(EXIT_FAILURE);
  }
  fromsegdist(w1_encl,dist,h);
  for(int i=1;i<=dim;i++)
    dist[i]+=Ldelta[i];
}



void InclOde::incl_fromsegdist(interval w1_encl[],interval w2_encl[],
       SetWithTail & wy, interval dist[],interval h, interval m[][MAX_DIM+1])
   //  function computes a distance from segment the joinnig the end points
  //  of the "unperturbed ode" (i.e.  with fixed y_c)
  //   BUT EXPRESSES IT IN NEW COORDINATES m (new coordinates)
    //  - the matrix m=pcc->intv_m; new coordinates are given by y=m*(x-P)
    //    w1_encl,w2_encl - are given on input in old coordinates
    // useful in the section region, when other coordinates are used
{

  interval h0h=Ihull(interval(0),h);
  interval Ldelta[MAX_DIM+1];
  if(!incl_compute_Ldelta(h0h,w1_encl,w2_encl,wy,Ldelta)){
    cerr << "computation of LDelta failed" << endl;
      #if defined(KRAK) && defined(WIN95)
           waitBt();
       #endif
    exit(EXIT_FAILURE);
  }

  fromsegdist(w1_encl,dist,h,m);

  multAV(m,Ldelta,Ldelta,dim); //  Ldelta - is now expressed in new coordinates

  for(int i=1;i<=dim;i++)
    dist[i]+=Ldelta[i];

}

int  InclOde::incl_dissOneStep(const interval /*x0*/[],interval /*h */,interval xencl[],
                      interval xstep[])
  // on input: \varphi([0,h],x0) \subset xencl
  // On output in  <xstep> - the bounds for \varphi(h,x)_i
  //           obtained from the dissipativity are returned
{
  copyvector(xencl,xstep,dim);
  return 1;
}

void  InclOde::writeLDeltaMethod(ostream &report){
  const int buflen=80;
  char methodname[buflen];
  report << inclmethoddescr(methodname,buflen,this->Ldelta_method) << "   ";
  if(this->Ldelta_method== Ldelta_lognorm){
    char nname[buflen];
    report << norm_name(nname,buflen,this->Ldelta_norm);
  }
  report << endl;
}
// InclOde - end


////  Perturbed_Ode - begin

void Perturbed_Ode::init_perturbed_ode(Ode *ode,interval pinclusion_interval[],
   int pLdelta_method,  int plognormused)
{
  assignvalues(ode);
  pode=ode;

  strcpy(infostr,pode->infostr);
  dim=pode->dim;

  for(int i=1;i<=dim;i++){
    y_c[i]=0;
    y_c_double[i]=0;
  }

  copyvector(pinclusion_interval,inclusion_interval,dim);

  Ldelta_method=pLdelta_method; //  the method used in LDelta computation
//  Ldelta_method=Ldelta_lognorm; // based on logarithmic norm
// Ldelta_method= Ldelta_componentwiseestm; // based on component-wise estimates

 if(plognormused==0)
    Ldelta_norm=pode->lognormused;
  else Ldelta_norm=plognormused;

}


Perturbed_Ode::Perturbed_Ode()
{
  dim=0;
  pode=NULL;
  setdefaultvalues(); // sets some variables on the ODE level
}

Perturbed_Ode::Perturbed_Ode(Ode *ode,interval pinclusion_interval[],int pLdelta_method)
{
  init_perturbed_ode(ode,pinclusion_interval,pLdelta_method,ode->lognormused);
}


Perturbed_Ode::Perturbed_Ode(Ode *ode,interval pinclusion_interval[],int pLdelta_method,
  int plognormused)
{
  init_perturbed_ode(ode,pinclusion_interval,pLdelta_method,plognormused);
}



void  Perturbed_Ode::incl_field(const interval x[],SetWithTail & /*yestm*/,interval f[])
{
  pode->field(x,f);
  for(int i=1;i<=dim;i++)
    f[i]+=inclusion_interval[i];
}


interval  Perturbed_Ode::incl_vfield(int k,const interval x[],SetWithTail &/*yestm*/)
    //   vfield(k,x)=f_k(x) + inclusion_interval[k]
{
  return pode->vfield(k,x)+inclusion_interval[k];
}


void  Perturbed_Ode::field(const double x[],double f[])
{
  pode->field(x,f);
  for(int i=1;i<=dim;i++)
    f[i]+=y_c_double[i];
}

void  Perturbed_Ode::field(const interval x[],interval f[])
{
  pode->field(x,f);
   for(int i=1;i<=dim;i++)
    f[i]+=y_c[i];
}


interval  Perturbed_Ode::vfield(int k,const interval x[])
//   vfield(k,x)=f_k(x)
{
  return pode->vfield(k,x) + y_c[k];
}

double Perturbed_Ode::vfield(int k,const  double x[])
  //   vfield(k,x)=f_k(x)
{
  return pode->vfield(k,x)+y_c_double[k];
}

interval   Perturbed_Ode::dvfield(int k, int i,const interval x[])
// dvfield(k,i,x)=\frac{\partial f_k}{\partial x_i}(x)
{
  return pode->dvfield(k,i,x);
}

double  Perturbed_Ode::dvfield(int k, int i,const double x[])
  // dvfield(k,i,x)=\frac{\partial f_k}{\partial x_i}(x)
{
  return pode->dvfield(k,i,x);
}

interval Perturbed_Ode::d2vfield(int k, int i, int j,const interval x[])
  // d2vfield(k,i,j,x)=\frac{\partial^2 f_k}{\partial x_i  \partial x_j}(x)
{
  return pode->d2vfield(k,i,j,x);
}


double   Perturbed_Ode::d2vfield(int k, int i, int j,const  double x[])
   // d2vfield(k,i,j,x)=\frac{\partial^2 f_k}{\partial x_i  \partial x_j}(x)
{
   return pode->d2vfield(k,i,j,x);
}


int Perturbed_Ode::enclosure(const interval x[],interval h0, interval enclx[],int test)
// if true is returned then:
//    \varphi([0,h0],x) \subset enclx , for y=y_c
//  h0 - can be negative
   // test=0 -  when procedure fails then a message is send
  //             to a screen and files
  // test=1 -  failure does not result in message to screen
{
  return(pode->pert_enclosure(x,h0,enclx,1,y_c,test));
}

void Perturbed_Ode::c1enclosure(interval h,const interval xencl[],
                                 interval Vencl[][MAX_DIM+1] )
 // computation of enclosure for variational part
  //  \frac{\partial \varphi}{\partial x} ([0,h],x)=V([0,h],x)
  //  i.e starting from identity
  //might be used in the integration of 'deterministic' part with y=y_c
{
  pode->c1enclosure(h,xencl,Vencl);
}

int Perturbed_Ode::c1dissOneStep(const interval xencl[],interval h,
               interval Vencl[][MAX_DIM+1],interval Vstep[][MAX_DIM+1])
  // for dissipative ODE's in Vstep returned are  the bounds for
  //         \frac{\partial \varphi_i}{\partial x}_j(h,x0)
  //           obtained from the dissipativity, x0 is the initial condition
  //            for x, and xencl is such that \varphi([0,h],x0) \subset xencl
  //might be used in the integration of 'deterministic' part with y=y_c
{
  return pode->c1dissOneStep(xencl,h,Vencl,Vstep);
}



int  Perturbed_Ode::incl_enclosure(const interval x[],SetWithTail & yin,interval h0,
     interval enclx[],SetWithTail & encly,int nomessages/*=0 */)
// if true is returned then:
//    \varphi([0,h0],(x,yin)) \subset (enclx,encly)
//
 // for this end we want  the following condition:
   //    enclx \subset interior(Z)
   //  where: enclx=x + [0,h]*incl_vfield(Z)
//  yencl,yin - don't have the tail, but
//  on output yencl=inclusion_interval   - the perturbation to the ODE
//  also <inclusion_interval> has to be computed, if necessary .
//  In this implementation it is just a constant, but for Galerkin
//  projections <inclusion_interval> has to be upadated
// in this function

   // if nomessages==0 -  then when the procedure fails then a message is send
  //             to a screen and files
  // if nomessages==1 -  then the failure does not result in message to screen
{
   SetWithTail temp(dim,inclusion_interval,yin.tailexists,yin.ptail);
   encly=temp;
   return(pode->pert_enclosure(x,h0,enclx,1,inclusion_interval,nomessages));
}


void  Perturbed_Ode::fromsegdist(interval encl[],interval dist[],interval h)
 // the function assumes that both a set and its image are contained in
    // <encl> for time [0,h]
    //  then computes the distance from the segment connecting the end points and a point
    //  from trajectory (with a current y_c) - in this case without any perturbation
    //  on return dist[i] = 0 if x_i - coordinate is monotonic inside <encl>
    //            and is nonzero otherwise and equal to the above mentioned
    //            distance between segment and entire trajectory
    // useful in section region
{
  pode->pert_fromsegdist(encl,dist,h,1,y_c);
}


void  Perturbed_Ode::fromsegdist(interval encl[],interval dist[],interval h,
                    interval mac[][MAX_DIM+1])
    //  the function assumes that both a set and its image are contained in
    // <encl> for time [0,h]
    //  then computes distance from segment joining end points and a point
    //  from trajectory, BUT EXPRESSES IT IN NEW COORDINATES m (new coordinates)
    //  - the matrix pcc->intv_m; new coordinates are given by y=m*(x-P)
    //    <encl> - is given on input in old coordinates
    //  on return dist[i] = 0 if x_i - coordinate is monotonic inside <encl>
    //            and is nonzero otherwise and equal to the above mentioned
    //            distance between segment and entire trajectory
    // useful in the section region, when other coordinates are used
{
  pode->pert_fromsegdist(encl,dist,h,mac,1,y_c);
}


int  Perturbed_Ode::y_move(interval /*xencl*/[],SetWithTail &yencl,interval /*h */,
      SetWithTail & /*y*/, SetWithTail &yn)
{
  yn=yencl;
  return 1;
}

interval  Perturbed_Ode::delta_vf(int i,const interval /*x*/[],SetWithTail & /*estmy*/ )
 // computes i-th coordinate of  f_i(x,estmy) - f_i(x,y_c)   for x \in <x>
{
  return inclusion_interval[i] - y_c[i];
}


void  Perturbed_Ode::delta_field(const interval /*x*/[],SetWithTail & /*estmy*/,interval deltaf[])
    //   deltaf=  f(x,estmy) -f(x,y_c) for x \in <x>
{
  for(int i=1;i<=dim;i++)
    deltaf[i]=inclusion_interval[i] - y_c[i];
}


interval   Perturbed_Ode::incl_dvfield(int k, int i,const interval x[],
   SetWithTail & /*yestm*/)
  //    dvfield(k,i,x,yestm)=\frac{\partial f_k}{\partial x_i} (x,yestm)
{
  return pode->dvfield(k,i,x);
}






///////////////
SetWithTail::SetWithTail(int pdim,interval py[],int pwithtail,KuraShi_Tail* ptotail)
//  pwithtail  - 0 - means that we create SetWithTail, with NULL-tail
//               1 - otherwise, and ptail=ptotail
{
  dim=pdim;
  y=new interval[dim+1];
  for(int i=1;i<=dim;i++)
    y[i]=py[i];
  tailexists=pwithtail;
  if(tailexists)
    ptail=ptotail;
  else ptail=NULL;
  removetail=0;
}

SetWithTail::SetWithTail(int pdim,int pwithtail,KuraShi_Tail* ptotail)
//  pwithtail  - 0 - means that we create SetWithTail, with NULL-tail
//               1 - otherwise, and ptail=ptotail
{
  dim=pdim;
  y=new interval[dim+1];
  zero(y,dim);
  tailexists=pwithtail;
  if(tailexists)
    ptail=ptotail;
  else ptail=NULL;
  removetail=0;
}

SetWithTail::SetWithTail(int pdim)
{
  dim=pdim;
  y=new interval[dim+1];
  zero(y,dim);
  tailexists=0;
  ptail=NULL;
  removetail=0;
}

SetWithTail::SetWithTail()
{
  dim=0;
  y=NULL;
  tailexists=0;
  ptail=NULL;
  removetail=0;
}


SetWithTail::SetWithTail(const SetWithTail &s)
{
  removetail=0;
  dim=s.dim;
  y=new interval[dim+1];
  for(int i=1;i<=dim;i++)
    y[i]=s.y[i];
  tailexists=s.tailexists;
  if(tailexists){
    removetail=1;
    ptail=new KuraShi_Tail(*(s.ptail));
  }else ptail=NULL;
}


SetWithTail::~SetWithTail()
{
  delete [] y;
  if(removetail)
    delete ptail;
}

SetWithTail & SetWithTail::operator=(const SetWithTail &source)
{
  if(dim != source.dim){
    delete [] y;
    dim=source.dim;
    y=new interval[dim+1];
  }
  for(int i=1;i<=dim;i++)
    y[i]=source.y[i];
  if(tailexists){
    if(removetail) delete ptail;
  }
  removetail=0;
  tailexists=source.tailexists;
  if(tailexists){
    removetail=1;
    ptail=new KuraShi_Tail(*(source.ptail));
  }
  return *this;
}

void  SetWithTail::assigntail(KuraShi_Tail &T)
        // it substitues (copies) the current tail by T
        // and does all neccessary updates
{
  if(tailexists && removetail)
    delete ptail;

  tailexists=1;
  ptail=new KuraShi_Tail(T);
  removetail=1;
}


void SwT_hull(const SetWithTail &S1,const SetWithTail &S2,SetWithTail &S)
//  On output  S ; such that S_1 \cup S_2 \subset S
{
  if((S1.dim != S2.dim) || (S1.tailexists!=S2.tailexists)){
    cerr <<  "SwT_hull() - dim or tail don't match" << endl;
    #if defined(WIN95) && defined(KRAK)
      cerr << "press any key " << endl;
      waitBt();
    #endif
    exit(EXIT_FAILURE);
  }
  int pdim=S1.dim;
  interval *py;
  py=new interval[pdim+1];
  for(int i=1;i<=pdim;i++){
    py[i]=Ihull(S1.y[i],S2.y[i]);
  }
  int ptailexists=S1.tailexists;
  KuraShi_Tail *pptail;
  pptail=NULL;
  if(ptailexists){
    pptail=new KuraShi_Tail(0,0);
    KS_Tail_hull(*(S1.ptail), *(S2.ptail),*pptail);
  }
  SetWithTail tempS(pdim,py,ptailexists,pptail);
  S=tempS;
  delete [] py;
  delete pptail;
}

int commonpart(const SetWithTail &S1,const SetWithTail &S2,SetWithTail &S, int tailintersectiononly)
//  on output 1 is returned if S_1 \cap S_2 \nonempty
//             and S becomes an upper bound for  S1 \cap S2
//       obtaining the exact intersection is imposible for far tail
// otherwise 0 is returned and S is unchanged
//// if tailintersectiononly - we do it  for  the tails only
{
  int ret=1;
  if((S1.dim != S2.dim) || (S1.tailexists!=S2.tailexists)){
    cerr <<  "commonpart(SetWithTail) - dim or tail don't match" << endl;
    #if defined(WIN95) && defined(KRAK)
      cerr << "press any key " << endl;
      waitBt();
    #endif
    exit(EXIT_FAILURE);
  }
  int pdim=S1.dim;
  interval *py;
  py=new interval[pdim+1];
  if(!tailintersectiononly)
    ret=commonpart(S1.y,S2.y,py,pdim);
  else copyvector(S.y,py,pdim);

  if(ret){
    int ptailexists=S1.tailexists;
    KuraShi_Tail *pptail;
    pptail=NULL;
    if(ptailexists){
      pptail=new KuraShi_Tail(0,0);
      ret=commonpart(*(S1.ptail), *(S2.ptail),*pptail);
    }
    SetWithTail tempS(pdim,py,ptailexists,pptail);
    S=tempS;
    delete pptail;
  }

  delete [] py;
  return ret;
}


int takeintersectionSWT(int xdim,int tailevolution,const interval x[],const interval fx[],
           const SetWithTail &y,const SetWithTail &fy,interval  xnew[],SetWithTail & ynew,
           int *emptyxfx, int *emptyyfy)
//  On output : xnew= x \cap fx, ynew= y \cap fy
//         if(!tailevolution) then ynew=y
//         *emptyxfx=1 iff x \cap fx =\emptyyset
//         *emptyyfy=1 iff y \cap fy = \emptyset
// 1 is returned if the intersection is non-empty
// 0 - otherwise
{
  *emptyxfx=0;
  *emptyyfy=0;
  int ok=commonpart(x,fx,xnew,xdim);
  if(!ok) *emptyxfx=1;
  if(ok && tailevolution){
    int tailintersectiononly=1;  // this is a variable tell that
          // (if ==1) from y - only the tail is of interest for us
          //   for PDE it is allways only the tail the object of interest
          //   because y.y - contains just the Galerkin projection errors

    ok=commonpart(y,fy,ynew,tailintersectiononly);
    if(!ok) *emptyyfy=1;
  }
  return ok;
}

