
#include<iostream>
#include<fstream>
using namespace std;


#include"defo.h"
#include"func.h"
#include"multiint.h"
#include"zgfiles.h"
#include"zset.h"



void transformSection(double sec[],const LinCC *lincc)
//  the transformation of the section from <x>  to new coordinates <y>
//  y=M(x-P)
// section is given by \Sigma_{i=1} sec[i]*x[i] + sec[0]=0
//                           sec^t * x + sec[0]=0
// modified section is given by the following equations:
//  ((M^{-1})^t sec)^t * y + sec^t P + sec[0]=0
//  or which is the same:
//         sec^t * M^{-1} * y + sec^t P + sec[0]=0
{
  double secnew[MAX_DIM+1];
  secnew[0]=sec[0];
  for(int i=1;i<=lincc->dim;i++)
    secnew[0]+=sec[i]*(lincc->origin_shift)[i];

  for(int i=1;i<=lincc->dim;i++){
    double t=0;
    for(int j=1;j<=lincc->dim;j++)
      t+=sec[j]*(lincc->mInv)[j][i];
    secnew[i]=t;
  }

  for(int i=0;i<=lincc->dim; i++)
    sec[i]=secnew[i];

}


void transformSection(interval sec[],const LinCC *lincc)
//  the transformation of the section from <x>  to new coordinates <y>
//  y=M(x-P)
// section is given by \Sigma_{i=1} sec[i]*x[i] + sec[0]=0
//                           sec^t * x + sec[0]=0
// modified section is given by the following equations:
//  ((M^{-1})^t sec)^t * y + sec^t P + sec[0]=0
//  or which is the same:
//         sec^t * M^{-1} * y + sec^t P + sec[0]=0
{
  interval secnew[MAX_DIM+1];
  secnew[0]=sec[0];
  for(int i=1;i<=lincc->dim;i++)
    secnew[0]+=sec[i]*(lincc->origin_shift)[i];

  for(int i=1;i<=lincc->dim;i++){
    interval t=0;
    for(int j=1;j<=lincc->dim;j++)
      t+=sec[j]*(lincc->mInv)[j][i];
    secnew[i]=t;
  }

  for(int i=0;i<=lincc->dim; i++)
    sec[i]=secnew[i];

}

/////////


void C1Func::intersection(const interval x[],interval x_on_level[]) const
  // on output   x \\cap \\{z| f(z) ==0 \\} \subset x_on_level
       // used by Poincare maps (really needed in C1 computations)
       // the one here just copies <x> to <x_on_level>
{
  copyvector(x,x_on_level,dim);
}


/* ***************** LinearF -***************************** */

double LinearF::eval(const double x[]) const
{

  double res=ci[0];
  for(int i=1;i<=dim;i++)
    res+=ci[i]*x[i];

  return(res);
}

interval LinearF::eval(const interval x[]) const
{

  interval res=ici[0];
  for(int i=1;i<=dim;i++)
    res+=ici[i]*x[i];

  return(res);
}



interval LinearF::eval(const Set &x) const
{
  interval res;
  if(x.snumLohner()!=LOHNER1){
    res=eval(x.xc);
    //  QR-part
    res+=smarteval_vAv(ici,x.B,x.r,dim);
    if(x.doubletonRep())
      res+=smarteval_vAv(ici,x.C,x.r0,dim);
  }else{
    interval *xencl;
    xencl=new interval[dim+1];
    x.intvencl(xencl);
    res=eval(xencl);
    delete [] xencl;
  }
  return res;
}


void LinearF::doubletointerval()
{
  for(int i=0;i<=dim;i++)
    ici[i]=ci[i];
}

void LinearF::intervaltodouble()
{
  for(int i=0;i<=dim;i++)
    ci[i]=mid(ici[i]).left_b();
}



LinearF::LinearF()
{
  dim=0;
}


void LinearF::init(int pdim,const double pci[])
{
  dim=pdim;
  for(int i=0;i<=dim;i++){
    ci[i]=pci[i];
  }
  doubletointerval();
}


void LinearF::init(int pdim,const interval pci[])
{
  dim=pdim;
  for(int i=0;i<=dim;i++){
    ici[i]=pci[i];
  }
  intervaltodouble();
}

LinearF::LinearF(int pdim,const double pci[])
{
  init(pdim,pci);
}

LinearF::LinearF(int pdim,const interval pci[])
{
  init(pdim,pci);
}


void LinearF::gradient(const double /*x*/[],double grad[]) const
{
  for(int i=1;i<=dim;i++)
    grad[i]=ci[i];
}

void LinearF::gradient(const interval /*x*/[],interval grad[])  const
{
  for(int i=1;i<=dim;i++)
    grad[i]=ici[i];
}

void LinearF::intersection(const interval x[],interval x_on_level[]) const
       // on output  x \\cap \\{z| f(z) ==0 \\}  \\subset x_on_level
       // used by Poincare maps (really needed in C1 computations)
       // the one here just copies <x> to <x_on_level> if the function
       // is not a coordinate function
       //   otherwise is does the coresponding projection
{
  copyvector(x,x_on_level,dim);
  int selcoord=finddropcoord(ici,dim);
  if(selcoord){
    x_on_level[selcoord]=- ici[0]/ici[selcoord];
  }
}

void transformSection(LinearF& linfunc,const LinCC *lincc)
//  the transformation of the section from <x>  to new coordinates <y>
//  y=M(x-P)
{
  transformSection(linfunc.ici,lincc);
  linfunc.intervaltodouble();
}


/* **************** LinearF - END ************************/

Coord::Coord(int i/*=1*/)
{
  cno=i;
}


double Coord::eval(const double x[]) const
{
  return(x[cno]);
}

interval Coord::eval(const interval x[]) const
{
  return(x[cno]);
}


Radial::Radial(int i1/*=1*/,int i2/*=1*/)
{
  r1=i1;
  r2=i2;
}

double Radial::eval(const double x[]) const
{
  return(sqrt(pow(x[r1],2)+pow(x[r2],2)));
}


interval Radial::eval(const interval x[])  const
{
  interval ret;
  if(r1!=r2)
    ret=sqrt(power(x[r1],2)+power(x[r2],2));
  else ret=abs(x[r1]);
  return(ret);
}



int Map::iterate(int iter,const double x[],double fx[],double df[][MAX_DIM+1])
          // on output fx=f^iter(x), df=df^iter(x)
{
  double xs[MAX_DIM+1],xn[MAX_DIM+1];
  double dfnew[MAX_DIM+1][MAX_DIM+1];
  copyvector(x,xs,dim);
  int ret=1;
  int i=1;
  while(ret && (i<=iter)){
    ret=map(xs,xn,dfnew);
    if(ret){
      copyvector(xn,xs,dim);
      if(i==1){
        copymatrix(dfnew,df,dim);
      } else{
        ArrArr(dfnew,df,df,dim);
      }
    }
    i++;
  }
  if(ret)
    copyvector(xn,fx,dim);
  return ret;
}



int Map::iterate(int iter,const  double x[],double fx[])
//  on output fx=f^iter(x)
{
  double xs[MAX_DIM+1],xn[MAX_DIM+1];
  copyvector(x,xs,dim);
  int ret=1;
  int i=1;
  while(ret && (i<=iter)){
    ret=map(xs,xn);
    if(ret){
      copyvector(xn,xs,dim);
    }
    i++;
  }
  if(ret)
    copyvector(xn,fx,dim);
  return ret;
}


int FMap::iiterate(int iter,const interval x[],interval fx[],interval df[][MAX_DIM+1])
          // on output fx=f^iter(x), df=df^iter(x)
{
  interval xs[MAX_DIM+1],xn[MAX_DIM+1];
  interval dfnew[MAX_DIM+1][MAX_DIM+1];
  copyvector(x,xs,dim);
  estimates estm;
  int ret=1;
  int i=1;
  while(ret && (i<=iter)){
    ret=imapE(xs,xn,dfnew,&estm);
    if(ret){
      copyvector(xn,xs,dim);
      if(i==1){
        copymatrix(dfnew,df,dim);
      } else{
        ArrArr(dfnew,df,df,dim);
      }
    }
    i++;
  }
  if(ret)
    copyvector(xn,fx,dim);
  return ret;
}



int FMap::iiterate(int iter,const interval x[],interval fx[])
          // on output fx=f^iter(x)
{

  interval xs[MAX_DIM+1],xn[MAX_DIM+1];
  copyvector(x,xs,dim);
  int ret=1;
  int i=1;
  while(ret && (i<=iter)){
    ret=imap(xs,xn);
    if(ret){
      copyvector(xn,xs,dim);
    }
    i++;
  }
  if(ret)
    copyvector(xn,fx,dim);
  return ret;
}


int FMap::imapE( interval x[],interval fx[],estimates* estm)
  // on output fx=f(x)
{
  Set xic(this->dim,x);

  int ret=imapDE(xic,fx,estm);

  return ret;
}


int FMap::imap( interval x[],interval fx[])
  // on output fx=f(x)
{
  estimates estm;
  return imapE(x,fx,&estm);
}

int FMap::imapE(interval x[],interval fx[],interval df[][MAX_DIM+1],
          estimates *estm)
// dim(x)=dim, dim(fx)=dim; dim(df)=dim x dim
{

  Set xic(this->dim,x);



  int ret=imapDE(xic,fx,df,estm);

  return ret;
}

int FMap::imap(interval x[],interval fx[],interval df[][MAX_DIM+1])
{
  estimates estm;
  return imapE(x,fx,df,&estm);
}



int FMap::imapD( interval xcenter[], interval C[][MAX_DIM+1],interval r0[],
                 interval fx[])
       // on output fx=f(xcenter + C*r0)
{
  estimates estm;
  Set xic(this->dim,xcenter,C,r0);
  return  imapDE(xic,fx,&estm);
}


int FMap::imapD(Set &x, interval fx[])
       // on output fx=f(x)
{
  estimates estm;
  return  this->imapDE(x,fx,&estm);
}


int FMap::imapD( interval xcenter[], interval C[][MAX_DIM+1],interval r0[],
                 interval fx[],interval df[][MAX_DIM+1])
// on output fx=f(xcenter + C*r0)
  //             df - contains enclosure for df/dx
{
  estimates estm;
  Set xic(this->dim,xcenter,C,r0);
  return  imapDE(xic,fx,df,&estm);
}


int FMap::imapD( Set &x,interval fx[],interval df[][MAX_DIM+1])
       // on output fx=f(x)
       //             df - contains enclosure for df/dx
{
   estimates estm;
   return imapDE(x,fx,df,&estm);
}



/// Affine functions
AffineMap::~AffineMap()
{
  if(dataOn){
    delete [] d_b;
    delete [] intv_b;
    delete [] intv_A;
    delete [] d_A;
  }
}

void AffineMap::inittables(int pdim, interval piA[][MAX_DIM+1],interval pib[],
                 double pdA[][MAX_DIM+1],double pdb[])
{
  if(dataOn){
    delete [] d_b;
    delete [] intv_b;
    delete [] intv_A;
    delete [] d_A;
  }
  dataOn=0;
  dim=pdim;
  if(dim > 0){
    d_b=new double[dim+1];
    intv_b=new interval[dim+1];
    d_A=new doublematrix1[dim+1];
    intv_A=new intervalmatrix1[dim+1];
    dataOn=1;
    copyvector(pib,intv_b,dim);
    copyvector(pdb,d_b,dim);
    copymatrix(piA,intv_A,dim);
    copymatrix(pdA,d_A,dim);
  }
}


AffineMap::AffineMap(int pdim,interval A[][MAX_DIM+1],interval b[])
{
  if(pdim==0){
    dim=0;
    dataOn=0;
  }
  if(pdim>0){
    doublematrix1 *pdA;
    pdA=new doublematrix1[pdim+1];
    todouble(A,pdA,pdim);

    double *pdb;
    pdb=new double[pdim +1];
    todouble(b,pdb,pdim);

    inittables(pdim,A,b,pdA,pdb);

    delete [] pdb;
    delete [] pdA;
  }
}

AffineMap::AffineMap(int pdim,double A[][MAX_DIM+1],double b[])
{
  if(pdim==0){
    dim=0;
    dataOn=0;
  }
  if(pdim>0){
    intervalmatrix1 *piA;
    piA=new intervalmatrix1[pdim+1];
    tointerval(A,piA,pdim);

    interval *pib;
    pib=new interval[pdim +1];
    tointerval(b,pib,pdim);

    inittables(pdim,piA,pib,A,b);

    delete [] pib;
    delete [] piA;
  }
}

AffineMap::AffineMap(int pdim,interval A[][MAX_DIM+1])
{
  if(pdim==0){
    dim=0;
    dataOn=0;
  }
  if(pdim>0){
    interval *b;
    b=new interval[pdim+1];
    zero(b,pdim);
    doublematrix1 *pdA;
    pdA=new doublematrix1[pdim+1];
    todouble(A,pdA,pdim);

    double *pdb;
    pdb=new double[pdim +1];
    zero(pdb,pdim);

    inittables(pdim,A,b,pdA,pdb);

    delete [] pdb;
    delete [] pdA;
    delete [] b;
  }
}






AffineMap::AffineMap(interval c)
// one dimensional map c*x
{
  dataOn=0;

  int pdim=1;
  interval *b;

  b=new interval[pdim+1];
  zero(b,pdim);

  intervalmatrix1 *A;
  A=new intervalmatrix1[pdim+1];
  A[1][1]=c;

  doublematrix1 *pdA;
  pdA=new doublematrix1[pdim+1];
  todouble(A,pdA,pdim);

  double *pdb;
  pdb=new double[pdim +1];
  zero(pdb,pdim);

  inittables(pdim,A,b,pdA,pdb);

  delete [] A;
  delete [] b;
  delete [] pdA;
  delete [] pdb;

}




AffineMap::AffineMap(int pdim,double A[][MAX_DIM+1])
{
  if(pdim==0){
    dim=0;
    dataOn=0;
  }
  if(pdim>0){
    double *b;
    b=new double[pdim+1];
    zero(b,pdim);

    intervalmatrix1 *piA;
    piA=new intervalmatrix1[pdim+1];
    tointerval(A,piA,pdim);

    interval *pib;
    pib=new interval[pdim +1];
    zero(pib,pdim);

    inittables(pdim,piA,pib,A,b);

    delete [] pib;
    delete [] piA;
    delete [] b;
  }
}

int AffineMap::map( double x[],double fx[])
  // on output fx=f(x)
{
  multAV(d_A,x,fx,dim);
  for(int i=1;i<=dim;i++)
    fx[i]+=d_b[i];
  return 1;
}

int AffineMap::imap( interval x[],interval fx[])
  // on output fx=f(x)
{
  multAV(intv_A,x,fx,dim);
  for(int i=1;i<=dim;i++)
    fx[i]+=intv_b[i];
  return 1;
}

int AffineMap::imapDE(interval x[],interval C[][MAX_DIM+1],interval r[],
                                      interval fx[],estimates * /*estm*/)
{
  return imapD(x,C,r,fx);
}


int AffineMap::imapDE(Set &x,interval fx[],estimates * /*estm*/)
{
  return imapD(x,fx);
}




int AffineMap::imapDE(interval x[],interval C[][MAX_DIM+1],interval r[],interval fx[],
               interval df[][MAX_DIM+1],estimates * /*estm*/)
{
  return imapD(x,C,r,fx,df);
}


int AffineMap::imapDE(Set &x,interval fx[], interval df[][MAX_DIM+1],estimates * /*estm*/)
{
  return imapD(x,fx,df);
}

int AffineMap::map(double x[],double fx[],double df[][MAX_DIM+1])
{
  this->map(x,fx);
  copymatrix(d_A,df,dim);
  return 1;
}

int AffineMap::imap(interval x[],interval fx[],interval df[][MAX_DIM+1])
{
  this->imap(x,fx);
  copymatrix(intv_A,df,dim);
  return 1;
}


int AffineMap::imapD(interval x[],interval C[][MAX_DIM+1],interval r[],interval fx[])
// fx=f(x+C*r)
{
  intervalmatrix1* AC;
  AC=new intervalmatrix1[dim+1];

  interval *t;
  t=new interval[dim+1];

  ArrArr(intv_A,C,AC,dim);  // AC=A*C
  multAV(AC,r,t,dim); // t =A*C*r

  this->imap(x,fx);   //  fx=A*x + b

  for(int i=1;i<=dim;i++)
    fx[i]+=t[i];


  delete [] t;
  delete [] AC;
  return 1;
}


int AffineMap::imapD(Set &x,interval fx[])
{
  intervalmatrix1* C;
  C=new intervalmatrix1[dim+1];

  interval *r;
  r=new interval[dim+1];

  interval *xc;
  xc=new interval[dim+1];

  x.get_xc(xc);
  if(x.usesIntervalSet()){
    identity(C,dim);
 x.get_r(r);
  }
  if(x.doubletonRep()){
    x.get_Cmatrix(C);
  x.get_r0(r);
  }
  if(x.QuboidRep()){
    x.get_Bmatrix(C);
    x.get_r(r);
  }


  int ret= imapD(xc,C,r,fx);

  delete [] xc;
  delete [] r;
  delete [] C;

  return ret;
}


int AffineMap::imapD(interval x[],interval C[][MAX_DIM+1],interval r[],interval fx[],
               interval df[][MAX_DIM+1])
// fx=f(x+C*r)
{
  this->imapD(x,C,r,fx);
  copymatrix(intv_A,df,dim);
  return 1;
}


int AffineMap::imapD(Set &x,interval fx[],interval df[][MAX_DIM+1])
// fx=f(x)
{
  this->imapD(x,fx);
  copymatrix(intv_A,df,dim);
  return 1;
}


void mapcomparerignonrig(ostream &report,FMap *PMap,int mapiter,double xb[])
// compares rigorous and nonrigorous computation for map PMap at xb
{
  double fx[MAX_DIM+1];
  int ok=PMap->iterate(mapiter,xb,fx);
  if(!ok){
    char message[]= "unable to complete the double computation ";
    cerr << message << endl;
    if(report)
      report << message << endl;
    return;
  }
  interval *ix, *ifx;
  ix=new interval[PMap->dim+1];
  ifx=new interval[PMap->dim+1];
  tointerval(xb,ix,PMap->dim);
  if(PMap->iiterate(mapiter,ix,ifx)){
    if(report){
      report << endl << "the interval image : " << endl;
      saveAllCoord(report,ifx,PMap->dim,1);
    }
    int contained=MultiIn(fx,ifx,0,PMap->dim);
    if(contained){
      char message[]="fx is contained in interval image";
      cout << message << endl;
      if(report)
        report << message << endl;
    }else{
      char message[]="fx is not contained in interval image";
      cout << message << endl;
      if(report)
        report << message << endl;
    }
  }else{
    char message[]="unable to complete the interval computation ";
    cerr << message << endl;
    if(report)
      report << message << endl;
  }
  delete [] ix;
  delete [] ifx;
}

