#ifndef _inclpoin_h
#define _inclpoin_h
#include"diffincl.h"
#include"rigpoinc.h"
#include"section.h"


int incl_Lo_Poincare(InclOde& ode, InclPoincareParType & param, Set &x,
           SetWithTail & y0,const C1Func &section, int secdir,estimates  *estm,
           interval Px[],SetWithTail & Py,interval dp[][MAX_DIM+1],
           int usediffcoord,LinCC *coordch);
//  computes rigorously the Poincare map of x using the Lohner
//     algorithm   for a differential inclusion <ode> x' \in ode.f(x) + y
//   where y0 is an initial value for y - for true differential inclusions
//    this parameter can be ignored, but it is important for PDEs with
//     the  evolving tail


//  1 if returned, when the Poincare map can be computed and the return time
//          is <= tmax,
//     then P(set(numLohner,x0,C0,r0)) \subset Px  (see below for the description
//     of the representation)
//    Py - on output is the value for the y-Poincare map, usually can be ignored
//              , but it is important for PDEs with th tail  evolving
//  equation for section is given by  section.eval(x)=0
//  dp - the variational matrix just after passing the section - Lohner matrix
//         C - for controlling a Lipschitz part of error, used elsewhere
//             for diagnosing properties, sooner or latter this type of information
//         shall become part of the estm - class, which will collect what is
//         needed for a particular computations
//

//  usediffcoord - tells if you different coordinates in the section region
//  coordch  - coordinates in which we express the result <Px>
//     (these are new coordinates), dp - is always expressed in OLD coordinates

// on OUTPUT :   on output Px  \times Py - is an enclosure of the image
//                usually Py - if this just perurbation of an ODE is not interesting for us
//                 but for PDE with an evolving tail - here is the info about tail



class InclPoincareMap
{
  protected:
   int destroy_Insection,destroy_Outsection; // is 1, when the constructor created the section
                     //  and then the destructor has to destroy it (remove it
                     // from the memory)
  public:
    int dim;
    char infostr[40];
    InclPoincareParType param;

    InclOde *pode;

    GeneralLinearSection *pInsection,*pOutsection; // describes the domain and range respectively

    int secdir;  // UP  or DOWN

    int virtual imap( Set &x,SetWithTail & y,interval fx[],SetWithTail & fy,
       estimates* estm);
  // on output f(x \times y) \subset  fx \times fy

    int virtual imap( Set & x,SetWithTail & y, interval fx[],SetWithTail & fy);
       // on output fx \times fy =f((x \times y)

    int virtual imapCC(Set &x,SetWithTail & y,interval fx[],SetWithTail & fy,
                 int coordOnInput,LinCC* inputcoordch,int coordOnOutput,
                 LinCC* outputcoordch);
    int virtual imapCCE(Set &x, SetWithTail & y,interval fx[],SetWithTail & fy,
               int coordOnInput,LinCC* inputcoordch,int coordOnOutput,
               LinCC* outputcoordch, estimates *estm);
       //   on output: f((x + C*r0) \times y) = fx \times fy
       // usually this is Poincare map
       //  It makes sense sometimes to play with different coordinate
       //  systems
       //   if (coordOnInput) then the output (x-variable) is expressed
       //   using the other coordinate  (linear) system  given by inputcoordch
       //
       //   if (coordOnOutput) then fx is given in coordinates <outputcoorch>
       // this function  assumes that the coordinates on section z[1]=0
       //  are (z[2],...) - where z - are the coordinates in this->psection->plincc


    double isettimestep(double ph);
    //  returns  previous timestep for interval computations , and sets timesptep=ph


    double igettimestep();

    double settmax(double ptmax);
    double gettmax();


    int isetorder(int porder);
  // returns previous value of order; and set order=porder and orderdf=order
  // interval computations


    int igetorder();

    InclPoincareMap(InclOde *ppode,GeneralLinearSection *ppsection,int psecdir,
        const InclPoincareParType &pparam,int pdestroy_section=0);


    InclPoincareMap(InclOde *ppode,GeneralLinearSection *ppInsection,
                   GeneralLinearSection *ppOutsection,int psecdir,
                   const InclPoincareParType &pparam,
                   int pdestroy_Insection=0,int  pdestroy_Outsection=0);



    InclPoincareMap();
    // default constructor - for heirs to be created without problems

    virtual ~InclPoincareMap();
    // destructor

  private:


};

typedef InclPoincareMap *pInclPoincareMap;


int inclPoinMap_map_divide(InclPoincareMap& inclPMap,int divisiontable[],
       int divdim, interval ixb[],interval C[][MAX_DIM+1],interval r0[],
       int numLohner,SetWithTail & y,interval fx[],SetWithTail & fy,int coordOnInput,
       LinCC* inputcoordch,int coordOnOutput,LinCC* outputcoordch,estimates *estm);
// the function computes pmap.map(ixb + C*r0)
//  for this computation   the set ixb + C*r0  is first divided
//     into \Pi_{1<= i <=divdim}  divisiontable[i] parts
 //  the i-th direction in <r0> is divided into divisiontabl[i] equal parts




void incl_Lo_orbit(InclOde& ode, InclPoincareParType &param,long num_time_steps, Set &x,
           SetWithTail & y0,const char comment[], double firstbias,
           bool savetraj, const char *report,bool saveestm,const char *repestm,bool savediam,
            ostream &outdiam);
//  computes rigorously the trajectory of x \oplus y0 using the Lohner
//       algorithm   for a differential inclusion <ode>
//       where y0 is an initial value for y - for true differential inclusions
//    this parameter can be ignored, but it is important for PDEs with
//       the evolving tail
//       num_time_steps - the number of time steps ; ignored if 0,
//       report - the name of file where the trajectory is saved
//       repestm - the name of file where the estimates along the trajectory
//                      are saved
//       outdiam - the file where we write the diameters


class InclEventManAlongTrajType{
  public:
    bool virtual sucessCondition(interval t,InclOde& ode, interval h,const Set &x,
           const interval xd[],const SetWithTail & y,const Set &xn,
           const interval xdn[], const SetWithTail &yn,const interval xencl[],
           const SetWithTail &yencl,const  Set &xs,const interval xds[],
           const interval yc[],interval w1_encl[],interval rem[]){return false;}
    bool virtual showInfo(interval t,InclOde& ode, interval h,const Set &x,
           const interval xd[],const SetWithTail & y,const Set &xn,
           const interval xdn[], const SetWithTail &yn,const interval xencl[],
           const SetWithTail &yencl,const  Set &xs,const interval xds[],
           const interval yc[],interval w1_encl[],interval rem[])=0;

    virtual double nextCheckCondition(double current_t){return 0;};  // if t >  nextCheckCondition()
                 // then the verification of    <sucessCondition>  should be attempted

    virtual double nextShowInfo(double current_t)=0; // if t >  nextShowInfo()
                 // then the verification of    <sucessCondition>  should be attempted

};


bool incl_Lo_comp_traj(InclOde& ode,const InclPoincareParType &param, Set &x,
           SetWithTail & y0, InclEventManAlongTrajType & trajHandler );
//  computes rigorously the trajectory of x \oplus y0 using the Lohner
//       algorithm   for a differential inclusion <ode>
//       where y0 is an initial value for y - for true differential inclusions
//    this parameter can be ignored, but it is important for PDEs with
//       the evolving tail
//  Output:
//    returns true - when the condition from trajHandler   is satisfied
//              when the computation of the trajectory ended
//            false - otherwise


#endif /* _inclpoin_h */
