
#include<iostream>
#include<fstream>
#include<cstdlib>
using namespace std;



#include"defo.h"
#include"faq.h"
#include"fnames.h"
#include"galks.h"
#include"zgfiles.h"
#include"multiint.h"
#include"section.h"
#include"oneshot.h"
#include"perfixp.h"
#include"kserr.h"
#include"rigkserr.h"
#include"inclpoin.h"
#include"kspoinc.h"
#include"ksinclpo.h"
#include"ksperiso.h"


// #define KSPERPAP    - this is for the programm accompying the paper 'Rigorous numerics ... III'
//        it is used to restrict a bit several parameter choices
//    it should be (un)defined   in defo.h


int cap_inflate_hs_w_tail(int xdim,int withtail,int tailevolution,interval x[],interval fx[],
           SetWithTail &y,SetWithTail &fy,interval  xnew[],SetWithTail & ynew,
           int *emptyxfx, int *emptyyfy)
// we take a 'carreful' intersection (with some inflation)
// dim(x)=dim(fx)=dim(xnew)=xdim
// 1 - is returned when the intersection is nonempty ;
//     in this case <xnew> is subset of x and contains x \cap fx
//                  <ynew> is subset of y and contains y \cap fy
// 0 - otherwise
{
  int ok;
  if(withtail)
    ok=takeintersectionSWT(xdim,tailevolution,x,fx,y,fy,xnew,ynew,
                                               emptyxfx,emptyyfy);
  else{
    *emptyyfy=0;
    ok=commonpart(x,fx,xnew,xdim);
    if(!ok) *emptyxfx=1;
  }

  double cmix=5e-2;
  double cmixfartail=5e-2;
  if(ok){
    for(int i=1;i<=xdim;i++){
      inflate_mix_intervals(x[i],xnew[i],xnew+i,cmix);
    }
    if(withtail && tailevolution){
      // now the same for the near tail
      interval t;
      for(int i=y.ptail->m+1;i<=y.ptail->big_m;i++){
        inflate_mix_intervals(y.ptail->a[i],ynew.ptail->a[i],&t,cmix);
        ynew.ptail->a[i]=t;
      }
      // now we take a look at the far tail
      interval yMp1=(*(y.ptail))(y.ptail->big_m+1);
      interval ynewMp1=(*(ynew.ptail))(y.ptail->big_m+1);
      inflate_mix_intervals(yMp1,ynewMp1,&t,cmixfartail);
      ynew.ptail->newfartail(t,ynew.ptail->s);
    }
  }
  return ok;
}





int corr_based_on_diameters(int mustbeinx,int xdim,int withtail,int tailevolution,
           interval x[],interval fx[],SetWithTail &y,SetWithTail &fy,
           interval  xnew[],SetWithTail & ynew)
// we make we change the candidate set taking into account
//  the sizes of fx and fy
// if withtail==0 then we ignore the tail
//   if tailevolution==0 then we set ynew=y
// dim(x)=dim(fx)=dim(xnew)=xdim
// 1 - is returned if  new guesses has been defined (in xnew and ynew)
// 0 - otherwise
//  if mustbeinx - then we make sure that on output xnew \subset x
//                                        and ynew \subset y
///////////
/// mustbeinx==1 should be used when we already have all correct inclusions
//   and we want to have new x,y which will be slightly larger that
///   x \cap fx, y\cap fy; but be still contained in x,y
//   so that the new x,y will be set on which also the inclusion from
//       Poincare map is satisfied
{
  int ret=1;
  double cmix=2e-1;
  ynew=y; // just to initialize the dimensions

  for(int i=1;i<=xdim;i++){
    shift_mod_interval(mustbeinx,x[i],fx[i],xnew+i,cmix);
  }

  if(withtail){
    ynew=y;
    if(tailevolution){
      interval t;
      for(int i=y.ptail->m+1;i<=y.ptail->big_m;i++){
        shift_mod_interval(mustbeinx,y.ptail->a[i],fy.ptail->a[i],&t,cmix);
        ynew.ptail->a[i]=t;
      }
      // now we take a look at the far tail
      interval yMp1=(*(y.ptail))(y.ptail->big_m+1);
      interval fyMp1=(*(fy.ptail))(y.ptail->big_m+1);
      shift_mod_interval(mustbeinx,yMp1,fyMp1,&t,cmix);

      int snew=fy.ptail->s;
      if(mustbeinx && (y.ptail->s > fy.ptail->s)) snew=y.ptail->s;
      ynew.ptail->newfartail(t,snew);
    }
  }
  return ret;
}



int gen_hset_with_tail_cap_infl(ostream &report, int i,int iter,int xdim,
     int withtail,int tailevolution,interval x[],interval fx[],
            SetWithTail &y,SetWithTail &fy,interval  xnew[],SetWithTail & ynew)
// the generation of <xnew>, <ynew> new candiate set for the h-set with tail
// for the strategy  <gen_new_intersect>
// xdim=dim(x)=dim(fx)=dim(xnew)
// On output - 1 is always returned
// if the intersection (x \oplus y) \cap (fx \oplus fy) is nonempty ;
//     in this case <xnew> is subset of x and contains x \cap fx
//                  <ynew> is subset of y and contains y \cap fy
//  if this intersection is empty - then we shift xnew \oplus ynew
//                     is centered at center of fx \oplus fy
//          and in each direction, i, the diameter is based on some convex combination
//               diameters of x[i] and fx[i]
{
  int emptyxfx,emptyyfy;
  int ok=cap_inflate_hs_w_tail(xdim,withtail,tailevolution,x,fx,y,fy,xnew,ynew,
            &emptyxfx,&emptyyfy);

  if(!ok){
    if(emptyxfx){
      cout << "the intersection of x and fx  is empty " ;
      if(i>0){
        cout << "  for " << i << "-th set in the image";
      }
      cout << endl;
      report << endl <<  "the intersection of x and fx is empty ";
      if(i>0){
        report << "for " << i << "-th set in the image ";
      }
      report << endl;

      write_empty_intersection(report,x,fx,xdim,"x","fx");
    }
    if(emptyyfy && withtail){
      if(i>0){
        report << endl << "problem with " << i << "-th set in the image" << endl;
      }
      report << "y \\cap fy = empty " << endl;
      cout << "the intersection of y and fy is empty (tails) ";
      if(i>0){
        cout <<" for "<< i << "-th set the image ";
      }
      cout  << endl;
    }

    if(emptyxfx || (emptyyfy && withtail)){
      if(corr_based_on_diameters(0,xdim,withtail,tailevolution,x,fx,y,fy,xnew,ynew)){
        ok=1;
        report << "iter = " << iter ;
        if(i>0){
         report << " set=" << i ;
        }
        report << "  the shift of the h-set " << endl;
        cout << " shifting of the " ;
        if(i>0) cout << i  << "-th ";
        cout << "h-set " << endl;
      }
    }
  }
  return ok;
}

int genhset_from_point(ostream &/*report*/,int iter,int xdim,
         int withtail,int tailevolution,interval x[],interval fx[],
                SetWithTail &y,SetWithTail &fy,interval  xnew[],
                 SetWithTail & ynew)
// the generation of <xnew>, <ynew> new candiate set for the h-set with tail
// for the strategy  <gen_from_point>
//  xdim=dim(x)=dim(fx)=dim(xnew)
// if (iter==1) then xnew,ynew are obtained by the inflation
//             of fx and fy respectively
// if iter >0 then for each direction (both in main variables and tail)
//  if fx[i]\subset x[i], then on output we have fx[i] \subset  xnew[i] \subset x[i].
//               xnew[i] is obtained through 'intelligent' intersection
//                     of x[i] and fx[i]
///   otherwise xnew[i] is set to fx[i] inflated
{
  int ret=1;
  if(iter==1){   // we assume that the initial data was a point
                 // hence we basically inflate the image
    double inflcoef=1.5;
    double inittailinflation=3.0;
    copyvector(fx,xnew,xdim);
    ArrInflate(xnew,inflcoef,xdim);
    if(withtail){
      ynew=fy;
      if(tailevolution){
        ynew.ptail->inflatediam(inittailinflation);
      }
    }
  }else{ // if we have an inclusion  fx[i] \subset x[i]  for some direction,
         // then fx[i[ \subset xnew[i] \subset x[i] and xnew[i]
         //            is obtained through 'intelligent'
         //       intersection ; the same holds for the tail
         // when there is no inclusion then we simply inflate fx[i]
    double cmix=3e-1;
    for(int i=1;i<=xdim;i++){
      if(fx[i].subset(x[i])){
        int must_be_in_in_x=1;
        shift_mod_interval(must_be_in_in_x,x[i],fx[i],xnew+i,cmix);
      } else{
        xnew[i]=inflate(fx[i],1+cmix);
      }
    }
    if(withtail){
      ynew=fy;
      if(tailevolution){
        interval t;
        for(int i=y.ptail->m+1;i<=y.ptail->big_m;i++){
          if((fy.ptail->a[i]).subset(y.ptail->a[i])){
            shift_mod_interval(1,y.ptail->a[i],fy.ptail->a[i],&t,cmix);
          }else{
            t=inflate(fy.ptail->a[i],1+cmix);
          }
          ynew.ptail->a[i]=t;
        }
        // now the far tail
        int snew=fy.ptail->s;
        interval yMp1=(*(y.ptail))(y.ptail->big_m+1);
        interval fyMp1=(*(fy.ptail))(y.ptail->big_m+1);
        if(fyMp1.subset(yMp1)){
          shift_mod_interval(1,yMp1,fyMp1,&t,cmix);
          if(y.ptail->s > snew) snew=y.ptail->s;
        }else{
          t=inflate(fyMp1,1+cmix);
        }
        ynew.ptail->newfartail(t,snew);
      }
    }
  }
  return ret;
}



int kstrajfile_exist(char* traj,int maxlentraj,const char *ksinfostr,int *dimintraj)
  // return 1 if there exists the file ksinfostr + 'orbit'+orbext
  //            which currently is '../data/ks/orbit.txt'
{
  int ok;
  goodcatname(traj,ksinfostr,maxlentraj);
  strcat(traj,"orbit");
  strcat(traj,orbext);
  if(fileexists(traj)){
//    cout << "-------------------------------" << endl;
//    cout << "enter the dimension in file" << traj << endl;
    ifstream tr(traj);
    tr >> *dimintraj ;
    cout << "the dim of data in [" <<  traj << "] is  " << *dimintraj << endl;
    ok=1;
  }
  else{
    #if defined(WIN95) && defined(KRAK)
      rootFrame.Clear();
    #endif
    cerr << "Trajectory file - [" << traj << "] not found " << endl;
    #if defined(WIN95) && defined(KRAK)
    cerr << "press any key " << endl;
    waitBt();
    #endif
    ok=0;
  }
  return ok;
}




int ks_rot13_boxes_galerrfromset(interval mu, const char* traj,double bias,
    int partition,int m,int big_m,double expcoeff,intervalmatrix1 *boxes,
     int *boxnum,interval galerr[],int num_last_coord, KuraShi_Tail *ks_tail )
//  On output: we obtain self-consistent bounds
//      boxes - the set W
//      ks_tail - the tail a^\pm_k for k > m
//      galerr[i]  - for i=1,\dots,m   the sizes of the influence of the tail
//                    onto f[i], hence Basic Differential Inclusion reads
/////////                 x[i]'=f_i[x] + galerr
//  the function uses the projection onto the plane (1,3) - which is divided
//  into <partition>-sections - using the angular coordinate
{
  if(num_last_coord <1)
    num_last_coord=0;

  int ret=0;
  *boxnum=0;

  if(ks_rot13_createbounds(traj,bias,m-num_last_coord,partition,boxnum,boxes,
        expcoeff)){
    bool isol_ok=false;

    interval* tail;
    tail=new interval[big_m+1];

    interval c;
    int s;
    int iternum=10;
    rigkserr(0,mu,boxes,*boxnum,m-num_last_coord,big_m,&isol_ok,galerr,tail,
                  &c,&s,iternum);


    ret=isol_ok;
    if(ret){
      if(num_last_coord>0){ // we extend boxes to <m> dimensions
                                               // and recompute bounds
        for(int b=1;b<=*boxnum;b++){
          for(int i=m-num_last_coord + 1;i<=m ; i++)
            boxes[b][i]=tail[i];
        }
        rigkserr(0,mu,boxes,*boxnum,m,big_m,&isol_ok,galerr,tail,&c,&s,iternum);
        ret=isol_ok;
      }
    }
    if(ret)   ks_tail->newvalues(m,big_m,s,c,tail);
    delete [] tail;
  }


  return ret;
}

void gal_saveinitialcond(ostream &report,const interval ixb[],
           const interval r0[],int dim,SetWithTail &y)
{
  saveinitialcond(report,ixb,r0,dim);
  y.ptail->save(report,2,"Tail");  // 2 - in c + [-diam,diam]
}


void  gal_save_res(ostream &report,const interval fx[],const interval rcomp[],
   int dim,SetWithTail &fy,int writeratios)
// we write down to file fx \oplus fy.tail
// if writeratios then we additionally compare diameters of fx and rcomp
{
  report << endl;
  report << "P(...)=" << endl;
  saveAllCoord(report,fx,dim,1);
  report << endl;

  report << "diameters(P(..)):" << endl;
  writediam(report,fx,dim);
  report << endl;

  fy.ptail->save(report,2,"Tail of P(..)");



  if(writeratios){ //
    report << "Quotients: diam(P(..))/diam(r): " << endl;
    write_compar_quotients(report,fx,rcomp,dim);
  }
}




void  write_no_tail_inclusion(ostream &report,KuraShi_Tail *fyptail,
         KuraShi_Tail *yptail)
{
  report << "Problem with the tail inclusion" << endl;
  int powersok=1;
  if((yptail->c >0) && (fyptail->c >0)) { // non-empty far tail
    report << "(fy.ptail)->s=" <<  fyptail->s << "   (y.ptail)->s=" <<
     yptail->s << "  " ;
    powersok=(fyptail->s >= yptail->s );
    if(powersok)  report << " OK";
    else report << " problem with powers ";
    report << endl;
  }
  if(powersok){
    for(int i=fyptail->m+1; i<= fyptail->big_m+1;i++){
      if(!in((*fyptail)(i),(*yptail)(i),0)){
        if(diameter((*yptail)(i))>0){
          interval r=diameter((*fyptail)(i))/diameter((*yptail)(i));
          report << "no inclusion i="<<  i << "  fy[i]=" <<
     (*fyptail)(i) << "    y[i]=" << (*yptail)(i)  <<
            "  diam-ratio=" << r.right_b() << endl;
        }else{
          report << "no inclusion i="<<  i << "  fy[i]=" <<
     (*fyptail)(i) << "    y[i]=" << (*yptail)(i) << endl;
        }
      }

    }
  }
}

void  gal_simple_repproof_status(ostream &report,int locproved,
            int deterministic,int full_galerkinproof,int incl_proven)
{
  if(locproved){
    char detincl[40];
    strcpy(detincl,((deterministic)?"deterministic":"inclusion"));
    report << detincl << "  BROUWER - PROVED" << endl;
    cout << detincl << "  BROUWER - PROVED" << endl;
    if(full_galerkinproof){
      report << detincl << "  FULL CHECK - BROUWER - PROVED" << endl;
      cout << detincl << "  FULL CHECK - BROUWER - PROVED" << endl;
    } else{
      if(incl_proven){
        cout << "but still a full check of apriori bounds is needed " << endl;
        report << "but still a full check of apriori bounds is needed " << endl;
      }
    }
  } else
    report << "P(x) is not contained in x " << endl;
}


void gal_sectionDataSave(ostream &report,InclPoincareMap &InclPMap,bool useothercoord,const LinCC *coordch)
//  we save the section data from the proof of the existence of periodic orbit for Galerkin projections
//  in the text format
{
  report << "Section data: " << endl;
  GeneralLinearSection *lsec=new GeneralLinearSection(*(InclPMap.pInsection));
  if(useothercoord)
     lsec->new_coord_on_sec(coordch);

  lsec->savetext(report,0);   // 0 - indicates that only the doubles (rounded up values) are saved
  delete lsec;
}


int asymmetry_test(ofstream &report,InclPoincareMap *symPMap,interval ixb[],
    interval r0[],int numLohner,SetWithTail &y,int divtable[],int dimdiv,LinCC *coordch)
//  ixb + r0 - is the bound for periodic orbit
// we return true if symMap(ixb+x0 + y.ptail) \cap (ixb + r0 + y.ptail)=\emptyset
{
  int retval=0;
  int useothercoord=1;
  estimates estm;
  intervalmatrix1* C;
  C=new intervalmatrix1[symPMap->dim+1];
  identity(C,symPMap->dim); // this have to be the identity

  interval *fx;
  fx=new interval[symPMap->dim+1];

  SetWithTail fy(y);

  cout << "Checking the asymmetry condition " << endl;

  time_t before,after;
  before=time(NULL);

  report << endl <<  "The verification of the asymmetry " << endl;

  int ok=inclPoinMap_map_divide(*symPMap,divtable,dimdiv,ixb,C,r0,numLohner,y,fx,fy,
                      useothercoord,coordch,useothercoord,coordch,&estm);

  if(ok){
    after=time(NULL);
    double compt_time=difftime(after,before);
    if(compt_time > 1){
      report << "  computation time="  << compt_time << "  sec " << endl;
      if(compt_time > 2){
        cout << "  computation time="  << compt_time << "  sec " << endl;
      }
    }
  }else{
    report << "not computed " << endl;
  }
  gal_save_res(report,fx,NULL,symPMap->dim,fy,0);


  if(ok){
    interval *x;
    x=new interval[symPMap->dim+1];

    interval *cp;
    cp=new interval[symPMap->dim+1];

    for(int i=1;i<=symPMap->dim;i++)
      x[i]=ixb[i]+r0[i];

    if(commonpart(x,fx,cp,symPMap->dim)){
      cout << "asymmetry test not conclusive" << endl;
      report <<  "asymmetry test not conclusive" << endl;
    }else{
      cout << "asymmetry test passed" << endl;
      report <<  "asymmetry test passed" << endl;
    }

    delete [] cp;
    delete [] x;
  }

  delete [] fx;
  delete [] C;
  return retval;
}



////////////// PROOF OF AN ATTRACTING PERIODIC ORBIT ////////////////
int galsimpleintervaliteration(FPoincareMap &PMap,InclPoincareMap &inclPMap,
       Galerkin_Proj_KuraShi &galode,  double xb[],double delta[],int numLohner,
        SetWithTail &y0,ofstream &report,int coord,int dimdiag,
        int iternum,int do_divisions,int testasymmetry,InclPoincareMap *symPMap)
// xb - a good approximation of the attracting fixed point
// iternum - the number of iterates
// coord - the type of coordinates
//  diagdiam - the dimension used in some diagonalization process used
//             to construct 'good' coordinates
//  do_divisions - if ==1, then starting from the second iterate the domain
//              is divided into parts for the computations
// this function takes the set N=xb + \\Pi_i[-delta[i],delta[i]]
// it computes P(N) ; for the next iterate N=P(N) \\cap N and so on
//// returns 1 - when a set N is found such that P(N) \subset N
///// it must be that <inclPMap> - is pointing onto <galode> as an the inclusion
//   probably one can do it better from the formal point of view, but
//  I need this to  have an access to some functions in <galode>
//  testasymmetry - if 1 we check whether  x \cap symPMap(x)=\emptyset
//                where x the fixed point found;
//             it should be used when inclPMap - is the full Poincare map
//                     symPMap - is then half poincare
//           and the candidate orbit is nonsymmetric
{
  cout << " ------------------------ " << endl;
  #if defined(WIN95) && defined(KRAK)
    rootFrame.Clear();
  #endif

  int ok=1;


  double Dp[MAX_DIM+1][MAX_DIM+1];
  if(coord!=standardcoord) {

    cout << "Real computation of Dp " << endl;
    double trash[MAX_DIM+1];

    ok=PMap.map(xb,trash,Dp);
    if(ok){
      const int buflen=80;
      char buffer[buflen];
      report << "Dp (nonrigorous) computed using h=" <<   PMap.dgettimestep() <<
                  "  order=" << PMap.dgetorder() << "  method=" <<
                   num_method_name(buffer,buflen,PMap.dgetmethod()) << endl;
      report << "Diagonalization dimension=" <<  dimdiag << endl;
    }else{
      cout << "unable to compute DP " << endl;
      #if defined(WIN95) && defined(KRAK)
      waitBt();
      #endif
      return 0;
    }
  }


  interval ixp[MAX_DIM+1];
  tointerval(xb,ixp,PMap.dim);
  LinCC* coordch=NULL;
  if(ok)
    ok=buildcoordchange(coord,Dp,ixp,PMap.dim,&coordch,dimdiag);

  if(!ok){
    delete coordch;
    coordch=NULL;
    return  0;
  }

  report << "the starting point in standard section coordinates: " << endl;
  saveAllCoord(report,xb,PMap.dim);

  const int buflen=80;
  char buffer[buflen];
  report << osa_coordsname(buffer,buflen,coord) << endl;

  Set xs;
  interval ixb[MAX_DIM+1],fx[MAX_DIM+1];
  zero(ixb,PMap.dim); // in our coordinate frame xb is the origin

  intervalmatrix1 *C;
  C=new intervalmatrix1[PMap.dim+1];
  identity(C,PMap.dim); // C - must be an identity for the intersection
                       // r0 \\cap Pmap.map(x0+C*r0)
                       // to work correctly
  interval r0[MAX_DIM+1];
  for(int i=1;i<=PMap.dim;i++)
    r0[i]=interval(-delta[i],delta[i]);

  SetWithTail  y(y0),fy(y0.dim); // used to handle the tail


  double d=delta[1];
  cout << "the size of the set " << d << "   ";
  cout << "h=" << inclPMap.igettimestep() << "  ";
  cout << "order=" << inclPMap.igetorder() << endl;


  int strategy=gen_new_shift;
  if(d==0)
    strategy=gen_from_point;

  inclPMap.param.write(report);

  char buff[buflen];
  report << "strategy for gen. of new set = " << namehsetstrategy(strategy,buff,buflen) << endl;
  report.flush();


  estimates estm;

  int deterministic=0;
  int det_proven=0;
// the initial idea was to find first a good set for the deterministic case
//  by iteration, then use it as a starting set for the inclusion
//  BUT it never worked that way, basically there was no use of the set N
//  such set was always a kind of too tight


  int incl_proven=0; // tells that we have inclusion on main variables
  int full_galerkinproof=0; // full check
  int gennew_ic;


  int dimdiv=0;
  int divtable[MAX_DIM+1];

  int useothercoord=1;

  int iterno=1;
  while(ok && (iterno<=iternum)){

    gennew_ic=1;
    report << endl << "iterate=" << iterno << endl;
    gal_saveinitialcond(report,ixb,r0,PMap.dim,y);  // we assume that C=identity

    xs.newvalues(PMap.dim,numLohner,ixb,C,r0); // this is our starting set

    cout << "iterate=" << iterno <<  "/" << iternum <<  endl;

    writedivtable(cout,divtable,dimdiv);
    if(dimdiv>0) writedivtable(report,divtable,dimdiv);


    time_t before,after;
    before=time(NULL);

    if(deterministic)
      ok=PMap.imapDE(xs,fx,useothercoord,coordch,useothercoord,coordch,&estm);
    else{
      int withdivisions=0;
      if(do_divisions){
        if(iterno> 1){
          withdivisions=(double(iterno)/iternum > 0.5);
        }else withdivisions=do_divisions;
      }
      if(withdivisions){
        ok=inclPoinMap_map_divide(inclPMap,divtable,dimdiv,ixb,C,r0,numLohner,y,fx,fy,
                      useothercoord,coordch,useothercoord,coordch,&estm);
      }else{
        ok=inclPMap.imapCCE(xs,y,fx,fy,useothercoord,coordch,useothercoord,coordch,&estm);
      }
    }

    if(ok){
      after=time(NULL);
      double compt_time=difftime(after,before);
      if(compt_time > 1){
        report << (deterministic?"deterministic":"inclusion") << "  computation time="  << compt_time << "  sec " << endl;
        if(compt_time > 2){
          cout << "  computation time="  << compt_time << "  sec " << endl;
        }
      }
    }else{
      report << "not computed " << endl;
      continue;
    }
    int zerowidth=(r0[1].left_b()==r0[1].right_b());
    gal_save_res(report,fx,r0,PMap.dim,fy,!zerowidth);


    int numbigratios=0;
    double largestratio=0;
    if(!zerowidth){
      compute_ratios(r0,fx,PMap.dim,&numbigratios,&largestratio);
      cout << "Number of ratios > 1 is = " << numbigratios << endl;
      cout << "largest ratio = " << largestratio << endl;
    }else{
      numbigratios=PMap.dim;
    }

    if(!zerowidth){
      if(do_divisions && (iterno==1) && (numbigratios > 0)){
        dimdiv=numbigratios+1;
        for(int i=1;i<=dimdiv;i++)
           divtable[i]=2;
      }
    }


    // now I would like to take an intersection  of ixb+r0 \\cap fx
    interval x[MAX_DIM+1],cp[MAX_DIM+1];
    for(int i=1;i<=PMap.dim;i++)
      x[i]=ixb[i]+r0[i];


    int mvar_incl_proved=1;
    for(int i=1;((i<=PMap.dim) && mvar_incl_proved);i++){
      mvar_incl_proved=fx[i].subset(x[i]);
    }
    if(!mvar_incl_proved && (numbigratios==0)){
      write_no_inclusion(report,fx,x,PMap.dim,"fx","x");
    }




    int tail_incl_proved=1;
    if(!deterministic){
      tail_incl_proved=(*(fy.ptail)).subset(*(y.ptail));

      if(!tail_incl_proved){
        write_no_tail_inclusion(report,fy.ptail,y.ptail);
        cout << "no tail inclusion " << endl;
      }
    }



    int locproved=mvar_incl_proved && tail_incl_proved;

    if(locproved){
      if(deterministic){
        det_proven=1;
        gennew_ic=0;
      }else{
        if(!galode.fullcheckstatus()){
          incl_proven=1;
          gennew_ic=0;
          galode.fullcheckstatus(1); // new iteration will be with the full validation
        } else{
          incl_proven=1;
          full_galerkinproof=1;
          gennew_ic=(iterno!=iternum); // this means that in the last iterate
                       // we don't generate new h-set
        }
      }
    }


    gal_simple_repproof_status(report,locproved,deterministic,full_galerkinproof,
                              incl_proven);


    // the generation of a new set for the next iteration
    if(gennew_ic){
      interval *xnew;
      xnew=new interval[PMap.dim+1];
      SetWithTail ynew(y.dim);
      if(strategy==gen_from_point){

        ok=genhset_from_point(report,iterno,PMap.dim,!deterministic,
         galode.get_tailevolution(),x,fx,y,fy,xnew,ynew);

      }
      if(strategy==gen_new_intersect){
        ok=gen_hset_with_tail_cap_infl(report,0,iterno,PMap.dim,!deterministic,
                     galode.get_tailevolution(),x,fx,y,fy,xnew,ynew);
      }
      if(strategy==gen_new_shift){
        ok=corr_based_on_diameters(incl_proven,PMap.dim,!deterministic,
                          galode.get_tailevolution(),x,fx,y,fy,xnew,ynew);
      }

      if(ok){
        copyvector(xnew,ixb,PMap.dim);
        DecomposeIMulti(ixb,r0,PMap.dim);
        if(!deterministic)
          y=ynew;
      }
      delete [] xnew;
    }


    if(deterministic && det_proven)
      deterministic=0;


    iterno++;
  }
  //  if the proof went through, then the estimate for  fixed point is given
  // by ixb + r0 - in coordinates - <coordch> (new coordinates in <coordch>)
  //  the tail is in y.ptail


  if(ok){

    report << endl << "Estimates: " << endl;
    writeGlEstm(report,&estm,PMap.igetorder(),fabs(PMap.igettimestep()));


    if(incl_proven){

      // we set x=fx and y=fy - because we go the inclusion
      copyvector(fx,ixb,PMap.dim);
      DecomposeIMulti(ixb,r0,PMap.dim);
      y=fy;


      if(testasymmetry){
        asymmetry_test(report,symPMap,ixb,r0,numLohner,y,divtable,dimdiv,coordch);
      }

      // the computation of the 'error' of the periodic orbit found
      // 1. transform ixb+r to the ambient space coordinates
      // 2. compute an 'error' in various norms

      // Ad.1
      interval xfull[MAX_DIM+1],rfull[MAX_DIM+1];
      from_sec_to_ambient_space(inclPMap.pInsection,ixb,r0,xfull,
                    useothercoord,coordch);
      DecomposeIMulti(xfull,rfull,inclPMap.pode->dim);



      // Ad.2
      double dl2=galode.l2norm(rfull,y).right_b();
      double dh1=galode.h1norm(rfull,y).right_b();
      double dc0=galode.c0norm(rfull,y).right_b();
      double dc1=galode.c1norm(rfull,y).right_b();

      report << endl;
      report << "Center point for our bounds for periodic orbit: " << endl;
      saveAllCoord(report,xfull,inclPMap.pode->dim,0);

      y.ptail->save(report,2,"Tail");


      report << " Error estimates :" << endl;
      report  << "  diff. estm  L2=" << dl2 << "   H1=" << dh1 << "  C0="<<
      dc0 <<  "  C1=" << dc1 << endl;

      // the bounds listed above are obtained, by ignoring really the fact
      // that in the near-tail we have info on the a_k, namely the mid point
      // below we want to compute 'error' taking into account this info.
      SetWithTail ymid(y);
      for(int i=y.ptail->m+1;i<=y.ptail->big_m;i++){
        ymid.ptail->a[i]-=mid(ymid.ptail->a[i]); // we subtract the mid point
                        // in the near tail
      }
      dl2=galode.l2norm(rfull,ymid).right_b();
      dh1=galode.h1norm(rfull,ymid).right_b();
      dc0=galode.c0norm(rfull,ymid).right_b();
      dc1=galode.c1norm(rfull,ymid).right_b();
      report << " Error estimates with respect to the mid point in near tail:"
                    << endl;
      report  << "  diff. estm  L2=" << dl2 << "   H1=" << dh1 << "  C0="<<
      dc0 <<  "  C1=" << dc1 << endl;
    }

    // saving the section and coordinates:
    gal_sectionDataSave(report,inclPMap,useothercoord,coordch);

  }
  delete [] C;
  delete coordch;

  return incl_proven;
}

/// END - Brouwer by iteration



void incl_poinc_manysizes(int testfunc(FPoincareMap &,InclPoincareMap &,
       Galerkin_Proj_KuraShi &, double [],double [],int ,SetWithTail &,ofstream &,int ,
       int , int, int, int, InclPoincareMap *),
       FPoincareMap &PMap,InclPoincareMap &inclPMap,Galerkin_Proj_KuraShi
        &inclode,int numLohner,
       double xb[],double  maxdelta,double mindelta,
       SetWithTail &y0, ofstream &report,int coord, int dimdiag,int iternum,
       int do_division,int testasymmetry,InclPoincareMap *symPMap)
//  checks for various sizes that the Poincare map for PDE has a fixed point (testfunc)
//
//  xb - our approximate fixed point
//  y0 - the candidate for the tail
//  maxdelta  and mindelta - bounds on the size of the sides of N
//  coord - possible values:
//         - standardcoord
//         - orthogonalizedeigenvectors;
//         - eigenvectors;
//         - partialeigenbasis
//         - eigen_subspaces_ortogonalized
//  do_division - if ==1, then starting from the second iterate the domain
//              is divided into parts for the computations
//  testasymmetry - if 1 we check if   x \cap symPMap(x)=\emptyset
//                where x the fixed point found;
//             it should be used when inclPMap - is the full Poincare map
//                     symPMap - is then half poincare
//           and the candidate orbit is nonsymmetric
{
  double deltasize=maxdelta;
  double sizegrowth=0.5;
  double delta[MAX_DIM+1];

  int count=0;
  int atleastonceproven=0;
//  int lastproven=0;

  int theend=0;

  while(!theend){
    count++;
    for(int i=1;i<=PMap.dim;i++)
      delta[i]=deltasize;
     #if defined(WIN95) && defined(KRAK)
       rootFrame.Clear();
    #endif

    if(count >1){
      report << "----------------" << endl;
      report << "trying a new size of " << deltasize << endl;
    }
    if(testfunc(PMap,inclPMap,inclode,xb,delta,numLohner,y0,report,coord,dimdiag,iternum,
               do_division,testasymmetry,symPMap)){
      atleastonceproven=1;
 //     lastproven=1;
    }

    deltasize*=sizegrowth;
    if(deltasize < mindelta) theend=1;

  }
  if(atleastonceproven){
    cout << "something was proven" << endl;
    report << "THERE IS A PROOF FOR SOME DELTA" << endl;
  }
}

void info_nonsym_KSinclPoincareBrouwer(int uselocgalerr,int tailevolution)
{
  int aboutfixedgalerr=(!uselocgalerr && !tailevolution);
  int aboutlocgalerkinerr=uselocgalerr;
  int abouttailevolution=tailevolution;
  cout << "IMPORTANT FOR THE PROOF -  PARAMETER SETTINGS " << endl;
  if(aboutfixedgalerr){
    cout << "the Poincare map cannot be computed " << endl;
  }
  if(aboutlocgalerkinerr){
    cout << "the Poincare map cannot be computed " << endl;
  }
  if(abouttailevolution){
    cout << "mu=0.127 dim=11 order=5 delta=1e-4 h=1e-3 iterates=2 " << endl;
    #if !defined(KSPERPAP)
    cout << " section = 2(perpendicular) coord - normalized eigenvectors" << endl;
    #endif
    cout << "one iterate takes around 69 sec on my home PC(1.8 GHz) " << endl;
    cout << "---------------------------- " << endl;
    cout << "mu=0.1215 dim=13, order=6, h=4e-4, delta=5e-5 - 1e-4 iter=2, dim_diag=10 " << endl;
    #if !defined(KSPERPAP)
    cout << " section = 2(perpendicular) coord - normalized eigenvectors" << endl;
    #endif
    cout << "one iterate takes around 300 sec on my laptop 1.73Ghz " << endl;
    cout << "--------------------------------------" << endl;
  }
}

void info_symper_KSinclPoincareBrouwer(interval imu,int uselocgalerr,
        int tailevolution)
{
  int aboutfixedgalerr=(!uselocgalerr && !tailevolution);
  int aboutlocgalerkinerr=uselocgalerr;
  int abouttailevolution=tailevolution;
  cout << "IMPORTANT FOR THE PROOF -  PARAMETER SETTINGS " << endl;
  if(aboutfixedgalerr){
    cout << "fixed tail and global Galerkin errors (symmetric attracting periodic orbits)" << endl;
    cout << "for mu= 0.127, 0.1275, 0.128  dim=14 the proof goes thru with the  parameters: " << endl;
    cout << "order = 3, standard h, partition = 70   expansion coeff=1.1  M=42" << endl;
    cout << " section = 2(perpendicular), last coord. isol =3, " ;
    cout << " data from 16-dim orbit, expfactor=1.1 " << endl;
    cout << " in orthogonalized eigenvectors coordinates (dim diag=8)" << endl;
    cout << "delta = 2e-5 and number of iterates = 3" << endl;
    cout << "on my PC Pentium IV 1.8 MHZ BC5.02++ - one iterate takes around 3 minutes " << endl;
    cout << "---------------------------" << endl;
    cout << " for mu=0.125 we have a pair of complex eigenvalues, the proofs goes " << endl;
    cout << " in 15-dim - all other details are the same (or standard) " << endl;
    cout << " 4 iterates are needed, one iterate takes around 6 min on my home PC " << endl;
    cout << "---------------------------------" << endl;
    cout << "it is very difficult to obtain the proof when diam(mu)!=0 " <<
            endl;
    cout << "for mu=0.127 +[-,1]1e-7 I was able to get the proof - parameter ";
    cout <<  " as above, but order=5 and iter=4 " << endl;
    cout << "----------------------------------" << endl;
  }
  if(aboutlocgalerkinerr){
    cout << "fixed tail and local Galerkin errors (symmetric attracting periodic orbits)" << endl;
    cout << " for mu=0.127 dim=12, delta=2.5e-4, partition=50, expfactor=1.25 , 5 iterates; " << endl;
    cout << "   h=6e-4, order=4   data from 16-dim orbit" << endl;
    cout << " section = 2(perpendicular) " << endl;
    cout << "  in orthogonalized eigenvectors coordinates (dim diag=8)" << endl;
    cout << "around 35 sek per iterate (home PC)" << endl;
    cout << "for the reason I don't understand (other data?) in MPI linux machine   I've to take expcoeff=1.5 and 6 iterates " << endl;
    cout << "------------------------------" << endl;
  }
  if(abouttailevolution){
    cout << "with tail evolution  (symmetric attracting periodic orbits)" << endl;
    if(imu.right_b()>0.1){
      cout << " for mu=0.127 dim=10, delta=4e-4 (2e-4), 1 iterate; "
        << "   h=1e-3, order=4  (dim diag=9 " << endl;
      #if !defined(KSPERPAP)
      cout  << " section = 2(perpendicular) " ;
      cout << "  in eigenvectors coordinates (in ortho - 3iterates) "
                 << endl;
      #endif
      cout << "around 14 sec on my Laptop 1.73 GHz" << endl;
      cout << "it also works for dim=9 delta =4e-4 order=4 h=1e-3, iter=4 " << endl;

      cout << "--------" << endl;
    }
    if(fabs(imu.right_b() - 0.125) < 0.1){
      cout << "for mu=0.125 (a pair of complex eigenvalues) I've have the proof for: "
           << endl;
      cout << " dim=11, delta=2e-4, 2 iterates;    h=1e-3, order=5, dim diag=8   " << endl;
      #if !defined(KSPERPAP)
      cout <<  " section = 2(perpendicular) " ;
      cout << "  in eigenvectors coordinates " << endl;
      #endif
      cout << "-------------------" << endl;
    }
    if(imu.right_b() <= 0.1){
      cout << "for mu=0.032  " << endl;
      cout << "dim=23, delta=8e-5, order=5, dimdiag=12 " << endl;
      cout << "h=1.5e-4, iter=2 " << endl;
      #if !defined(KSPERPAP)
       cout << "section=2 (perpendicular), eigenvectors basis" << endl;
      #endif
      cout << "on my Laptop (1.73GHz) one iterate takes around 457 sec " << endl;
      cout << "-----------------" << endl;
    }
    if(diameter(imu) > 0){
      cout << "if diam(mu) > 0,  this should be rather very small " << endl;
      cout << "the lagerst interval we were able to get: "
         "for mu=0.127+1e-7[-1,1]" << endl;
      cout << "dim=10, delta=4e-4, h=8e-4, order=5, two iterates " ;
      cout <<  " dimdiag=9" << endl;
      #if !defined(KSPERPAP)
        cout <<  "  eigenvectors basis " << endl;
      #endif
      cout << "-----------------------"<< endl;
    }
  }
}



void ksinclcomp_description(ostream &report,KuraShi &kurashi,int symperorbit,
    int startsec,int partition,double expcoeff, int num_coord_iso,
    int tailevolution,int uselocgalerr,int sec_perpendicular,const double xs[],
    int pnumLohner)
{
  report << "============================" << endl;
  writeCompInfo(report);
  writedatetime(report);
  kurashi.writeparameters(report);
  if(symperorbit)
    report << "computation of the half-Poincare composed with symmetry " << endl;
  else
    report << "computation of the full Poincare map " << endl;

  report << "section= " << startsec << endl;
  report << "partition = " << partition  << endl;
  report << "expcoeff = " << expcoeff << endl;
  if(num_coord_iso > 0){
    report << " the number of last coordinates for which we guarantee "
              " an isolation " << num_coord_iso << endl;
  }

  if(tailevolution)
    report << "using the tail evolution " << endl;
  else{
    if(uselocgalerr)
      report << "using locally computed Galerkin errors " << endl;
    else
      report << "using global Galerkin errors " << endl;
   }

  if(sec_perpendicular)
    report << "perpendicular section " << endl;
  else report << "standard section " << endl;

  const int buflen=80;
  char buffer[buflen];
  report << "Lohner set - " << pnumLohner << "  " <<
        LohnerSetDescription(pnumLohner,buffer,buflen) << endl;

  report << "The starting point in ambient space standard coordinates: " << endl;
  saveAllCoord(report,xs,kurashi.dim);
}

void ksInclPoincPar(InclPoincareParType &param,KuraShi &kurashi)
// asks (and sets) for various parameters of numerical method
//       for differential inclusion
{
  param.h=changeh(param.h);
  int suggestedorder=kssugestedorder(kurashi.dim,(kurashi.imu).right_b(),param.h);
        // a minimum order to make the computation feasible without the dissipativity support

  if(suggestedorder < param.onesteppar.order) suggestedorder=param.onesteppar.order;
  param.onesteppar.order=changeorder(suggestedorder);

  #if !defined(KSPERPAP)
  selinclonestepmethod(&(param.onesteppar.onestepmethod),&(param.onesteppar.orderdf));
  #else
    param.onesteppar.onestepmethod=dfincl_C0simpleLo;  // standard C^0-Lohner plus the influence from the tail
  #endif

//  param.onesteppar.doC0intersection=askUseC0intersection();

}


void param_ghpp_comp(KuraShi &kurashi,int manyattempts, int tailevolution,
       int *startsec,int *sec_perpendicular,int *partition,
       double *expcoeff,int *pnumLohner,double delta[],int *emptytail,
       double *mindelta, double *maxdelta, int *iternum,int *set_partition,
      InclPoincareParType &param,int *coord,int *dimdiag)
// we assign interactively the parameters for KS-incl. proofs
{
  int dimpoinc=kurashi.dim-1; // the dimension of the Poincare map

  #if !defined(KSPERPAP)
  int sec_num=8;  // the number of possible sections
  section_choice("",sec_num,startsec);
  #else
    *startsec=2;
  #endif

  #if !defined(KSPERPAP)
  *sec_perpendicular=want_perpendicular_sec();
  #else
    *sec_perpendicular=1;   //  we use the perpendiuclar section to the flow
  #endif

  int askfor_part_expcoeff=1;
  if(tailevolution){ // it looks like it really don't matter
                     // how we set these parameters - it is enough
                     // just to take the large tail
    cout << "the standard initial tail from file will be used " << endl;
    *partition=50;
    *expcoeff=1.25;
    askfor_part_expcoeff=0;
  }

  #if !defined(KSPERPAP)
  if(askfor_part_expcoeff){
    cout << "partition (<=" << 100 << ") = " ;
    cin >> *partition ;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif


    cout << "the expansion coefficient in construction of tail (>1.0) ";
    cin >> *expcoeff;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }
  #endif

  cout << "this function takes the set N=xb + \\Pi_i[-delta[i],delta[i]] " <<endl;
  cout << "it computes P(N) ; for the next iterate N=P(N) \\cap N and so on" << endl;

  *emptytail=0;
   if(!manyattempts){
    #if !defined(KSPERPAP)
    inittabdelta(delta,dimpoinc);
    #else
    inittabconstdelta(delta,dimpoinc);
    #endif
    if(delta[1]==0)
      *emptytail=askemptytail();
  }else{
    cout << "max size = " ;
    cin >> *maxdelta ;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
    cout << "min size = " ;
    cin >> *mindelta ;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }

  cout << "number of iterates N=N \\cap f(N)=" ;
  cin >> *iternum;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  if(*iternum<1) *iternum=1;



  *set_partition=0;
  /*
  // do not remove it, it may be of some use later
  if(*iternum>0){
    cout << "do you want to partition the domain (from 2nd iterate) ?(1/0)" ;
    cin >> *set_partition;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
    if(*set_partition!=1) *set_partition=0;
  }
 */

  #if !defined(KSPERPAP)
  *pnumLohner= whichSetTypeChose();
  #else
  *pnumLohner=LOHNER5;  // Lohner doubleton, as in C^1-Lohner paper
  #endif

  ksInclPoincPar(param,kurashi); // we interactively set the time step, order (in param) etc


  #if !defined(KSPERPAP)
  *coord=osa_coord(*coord);
  #else
  *coord=eigenvectors;   //  we will diagonalize D Poinc for first *dimpoinc - coordinates
  #endif
  *dimdiag=dimpoinc;
  if(*coord!=standardcoord)  *dimdiag=howmanydiagcoord(dimpoinc);

}


int find_candidate_fp(KuraShi &kurashi,FPoincareMap *PMap,int iter_num,
          double xb[],double xs[],char traj[],double bias,double err)
// In xb and xs we return a candidate for the fixed point for <iter_num>-iterate
  //  xb - is on section and section coordinates
  //  xs - is xb, but it is expressed in coordinates of the ambient space
///// the fixed point is read from the file <traj> (which is on output contains
 //  the file name), <bias> is the time parameter along trajectory

{
  // a candidate for fixed point
  pointfromtrajfile(traj,xs,kurashi.dim,bias);   // reads a point from the trajectory
                           // this is our candidate for the fixed point


  cout << "moving point to section " << endl;
  int ok=PMap->transporttosection(xs,xb); //  <xb> is now  on section
                       // in section coordinates
  PMap->pInsection->sectiontofull(xb,xs);



  if(!ok){
    cerr << "unsuccessfull - probably a bigger tmax is  needed " << endl;
     #if defined(WIN95) && defined(KRAK)
       cerr << "press any key " << endl;
       waitBt();
    #endif
  }


  if(ok){
    cout << "nonrigorous search for an ATTRACTING fixed point " << endl;

    int max_iter=20;
    int iterused;
    ok=findattrfpMap(*PMap,xb,iter_num,max_iter,err,&iterused); //
        // for non-attracting we should use
        // newtonsearch
    PMap->pInsection->sectiontofull(xb,xs);

    if(!ok){
      cerr << "Fixed point not found" << endl;
      #if defined(WIN95) && defined(KRAK)
         cerr << "press any key " << endl;
         waitBt();
      #endif
    }
  }
  return (ok);
}

void ksinclpreparePMap(FPoincareMap **PMap,KuraShi &kurashi,int startsec,
          int symperorbit,InclPoincareParType &inclparam)
// creates the Poincare map , if symperorbit then this is half-poincare composed
// with the reflectional symmetry
{
 // interval rad13dir[MAX_DIM+1];
  interval *rad13dir;
  rad13dir=new interval[kurashi.dim+1];
  KuramotoRadialDir(rad13dir,startsec,kurashi.dim);
  dropcoord(rad13dir,2,rad13dir,kurashi.dim);

  Par_Poincare param;
  copyToParPoincare(inclparam,param);
  Par_Poincare pardouble=param;
  ksstablepar(kurashi.dim,kurashi.mu,param.h,param.onesteppar.order,
        &(pardouble.h),&(pardouble.onesteppar.order));
  pardouble.onesteppar.orderdf=pardouble.onesteppar.order;
  if(pardouble.onesteppar.order==4){
    pardouble.onesteppar.method=RUNGEKUTTA;
  }



  if(symperorbit){
    *PMap=createHPKS13(&kurashi,rad13dir,param,pardouble); // creates half poincare
          // composed with the symmetry changing signs on odd modes
         // with a section expressed in (a1,a3) coordinates  specified by <strtsec>
  } else{
    GeneralLinearSection *psection;
    psection=kscreateGLS13(kurashi.dim,rad13dir);

    int secdir=UP;
    int pdestroy_section=1;
    *PMap=new  FPoincareMap(&kurashi,psection,secdir,param,pardouble,pdestroy_section);
  }
  delete [] rad13dir;
}

void  ksincl_createinclpmaps(Galerkin_Proj_KuraShi **inclode,
          InclPoincareMap **inclPMap,InclPoincareMap ** asymtestPMap,
          FPoincareMap *PMap, InclPoincareParType &param,KuraShi &kurashi,
          interval **galerr, intervalmatrix1 **boxes,int *boxnum,KuraShi_Tail **ks_tail,
           int big_m,int partition,double bias, double expcoeff,
           int num_coord_iso,int uselocgalerr,int tailevolution,
           int symperorbit,int testasymmetry)
// prepares inclPMap and asymtestPMap for the proofs of the existence
// of periodic orbits, parameters for these maps are given in <param> and <PMap>
// num_coord_iso;  the number of coordinates for which we 'guarantee'
              // an isolation - important when using global tail
              // otherwise it is ignored
{
  const int maxlentrajf=80;
  char traj[maxlentrajf];
  int dimintraj;
  if(!kstrajfile_exist(traj,maxlentrajf,kurashi.infostr,&dimintraj)){ // this initiates
                          // the <traj> variable with the file name
    exit(EXIT_FAILURE);
  }

  *ks_tail=new KuraShi_Tail(kurashi.dim,big_m);
  *galerr=new interval[kurashi.dim +1];
  *boxes=new intervalmatrix1[partition+1];

  if(tailevolution) num_coord_iso=0;
  if(!ks_rot13_boxes_galerrfromset(kurashi.imu,traj,bias,partition,
    kurashi.dim,big_m,expcoeff,*boxes,boxnum,*galerr,num_coord_iso,*ks_tail)){
    exit(EXIT_FAILURE);
  }

  int Ldelta_method= Ldelta_componentwiseestm; // based on component-wise
                               // estimates
  int Ldelta_norm=NORM_MAX;
//  int method=TAYLORM;

  *inclode=new Galerkin_Proj_KuraShi(&kurashi,*galerr,kurashi.dim,Ldelta_method,Ldelta_norm,
        *boxes,*boxnum,*ks_tail,uselocgalerr,tailevolution);


  if(symperorbit){
    *inclPMap=new InclKSR_PMap(*inclode,PMap->pInsection,PMap->secdir,param);
  } else{
    *inclPMap=new  InclPoincareMap(*inclode,PMap->pInsection, PMap->secdir,param);
  }


  if(!symperorbit && testasymmetry){
    *asymtestPMap=new InclKSR_PMap(*inclode,PMap->pInsection,PMap->secdir,param);
  } else *asymtestPMap=NULL;
}


void  ksredefine_perpend_PMap(FPoincareMap **PMap,KuraShi &kurashi,double xs[],
             double xb[],int symperorbit)
// we change the PMap to a Poincare map  on the perpendicular section
//  at xs, dim(xs)=kurashi.dim
///// On output xb - is xs - expressed in new section coordinates for **PMap
///                dim(xb)=kurashi.dim-1=*PMap->dim
//
{
  GeneralLinearSection* perpendicularsection;

  perpendicularsection=createPerpendicularSection(kurashi.dim,xs,kurashi);
  // the creation of the section passing through the point <x>
  //  and perpendicular to the vector ode.vfield(x)
  // this section will be removed from memory by the destructor
  //  PMap

  zero(xb,kurashi.dim-1);    // in coordinates on the section  defined above
        // xb is at the origin

  Par_Poincare parinterval=(*PMap)->param.par_interval;
  Par_Poincare pardouble=(*PMap)->param.par_double;

  delete (*PMap);
  *PMap=NULL;

  int destroy_section=1;

  if(symperorbit)
    *PMap=new   KSR_PMap(&kurashi,perpendicularsection,UP,parinterval,pardouble,destroy_section);
  else
    *PMap=new  FPoincareMap(&kurashi,perpendicularsection,UP,parinterval,pardouble,
               destroy_section);

}

void symorbit_question(int *symperorbit, int *testasymmetry)
{
  cout << "for symmetric orbits  chose 1 , 0 -otherwise " ;
  cin >> *symperorbit;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  if(*symperorbit!=1) *symperorbit=0;
  *testasymmetry=0;
  if(!(*symperorbit)){
    cout << "do you want to perform the asymmetry test Yes-1, No -0 " ;
    cin >> *testasymmetry;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }
}


void KSinclPoincareBrouwer(KuraShi &kurashi)
// an attempt of full proof in infinite dimension of the existence
//  of the apparently attracting periodic orbit
// it WORKS for mu \approx 0.127, \mu=0.032, \mu=0.1215 (nonsymmetric)
//  the attractivity is not proved, just the existence
{
  int iterfixp=1;  // the fixed point (nonsymmetric) is for <iterfixp>-iterate

  int symperorbit=1;
  int testasymmetry=0;
  symorbit_question(&symperorbit,&testasymmetry);

  int manyattempts=0;    // tells whether we try various proof parameters - currently it is
                    //     the size only, later we will add the dimension
  #if !defined(KSPERPAP)
  cout << "try many sizes for the set N ? (1/0) ";
  cin >> manyattempts ;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  #endif


  int tailevolution=1; // tells if we allow for tail to evolve
  #if !defined(KSPERPAP)
  cout << "do you want to use the 'tail evolution' ? (1/0) " ;
  cin >> tailevolution;
  #if defined(WIN95) && defined(KRAK)
  cout << endl;
  #endif
  #endif

  int uselocgalerr=0; // tells if we are using local (computed locally along the trajectory)
                   //    or global galerkin errors good for the whole trajectory
  if(!tailevolution){
    cout << "do you want to use the local Galerkin errors ? (1/0) " ;
    cin >> uselocgalerr;
    #if defined(WIN95) && defined(KRAK)
    cout << endl;
    #endif
  }

  int emptytail=0; // emptytail, tells whether we start with empty tail in the first
  // iteration
  double delta[MAX_DIM+1];
  double maxdelta,mindelta;
  int numLohner;


  InclPoincareParType param;
  param.h=1e-3;
  param.h=kurashi.timestep(param.h);
  param.tmax=4.0;
  param.onesteppar.order=3;
  param.onesteppar.method=TAYLORM;
  param.onesteppar.doC0intersection=1;
  param.onesteppar.onestepmethod=dfincl_C0simpleLo;
  param.onesteppar.orderdf=param.onesteppar.order;

  int startsec=2;
  int sec_perpendicular=0;
  int partition=50;
  double expcoeff; //  it should be a slightly bigger than 1.0
//  int big_m=42;
  int big_m=3*kurashi.dim;
  int set_partition=1;
  int coord=eigenvectors;
  int iternum=3;

  int dimdiag=kurashi.dim-1;


   #if defined(WIN95) && defined(KRAK)
    rootFrame.Clear();
  #endif

  if(symperorbit)
    info_symper_KSinclPoincareBrouwer(kurashi.imu,uselocgalerr,tailevolution);
  else info_nonsym_KSinclPoincareBrouwer(uselocgalerr,tailevolution);


  param_ghpp_comp(kurashi,manyattempts,tailevolution,&startsec,
    &sec_perpendicular,&partition,&expcoeff,&numLohner,delta,&emptytail,&mindelta,&maxdelta,
       &iternum,&set_partition,param,&coord,&dimdiag);


  FPoincareMap *PMap;
  ksinclpreparePMap(&PMap,kurashi,startsec,symperorbit,param); // we create ('define') PMap

  double xs[MAX_DIM+1],xb[MAX_DIM+1];
     // xs and xb represent the same starting point, xb is on section
     //  (and expressed in section coordinates) xs contains the ambient space
     //   coordinates for xb

  const int maxlentrajf=80;
  char traj[maxlentrajf];
  //   initialization of the file name <traj>
  int dimintraj=16;
  if(!kstrajfile_exist(traj,maxlentrajf,kurashi.infostr,&dimintraj)){
    exit(EXIT_FAILURE);
  }


  double bias,err;
  if(manyattempts)
    err=mindelta/10.0;
  else err=delta[1]/10.0;
  if(err < 1e-10)
    err=1e-6;
  bias=1.0;

  if(!find_candidate_fp(kurashi,PMap,iterfixp,xb,xs,traj,bias,err)){
                //   this function finds the candidate for fixed point(attracting, by reading
                // a point form the trajectory in <traj>)  and also initiates the <traj> variable
    delete  PMap;
    PMap=NULL;
    return;
  }

  if(sec_perpendicular){
    ksredefine_perpend_PMap(&PMap,kurashi,xs,xb,symperorbit);
  }
  // until this moment we have defined a deterministic Poincare map
  // and we know the candidate for the fixed point <xb>- this will be center
  // of our initial condition


  // NOW WE INITIALIZE DIFFERENTIAL INCLUSIONS

  Galerkin_Proj_KuraShi *inclode;
  InclPoincareMap *inclPMap;
  InclPoincareMap *asymtestPMap;


  interval *galerr;
  int boxnum=0;
  intervalmatrix1 *boxes;
  KuraShi_Tail *ks_tail;

  int num_coord_iso=3; // the number of coordinates for which we 'guarantee'
              // an isolation - important when using global tail
  if(tailevolution) num_coord_iso=0;


  ksincl_createinclpmaps(&inclode,&inclPMap,&asymtestPMap,PMap,param,kurashi,&galerr,
     &boxes,&boxnum,&ks_tail,big_m,partition,bias,expcoeff,num_coord_iso,
             uselocgalerr,tailevolution,symperorbit,testasymmetry);


  char repname[80];
  strcpy(repname,catwork);
  strcat(repname,"ksihp");
  strcat(repname,orbext);
  ofstream  report(repname,ios::app);

  ksinclcomp_description(report,kurashi,symperorbit,startsec,partition,
          expcoeff,num_coord_iso,tailevolution,uselocgalerr,
          sec_perpendicular,xs,numLohner);


  report <<"Diff. inclusion simple limit cycle interval computation " << endl;

  int withtail=1;
  SetWithTail y0(kurashi.dim,galerr,withtail,ks_tail); // in fact it should'nt matter
             // what we plug-in as the second parameter (galerr),
             // because in y0 is the tail
             // is given by ks_tail, and y0.y=galerr - are the 'errors'
             // (depending on thetail,  but it makes sense to plug-in
             // here something that will be larger than all Galerkin errors


  if(emptytail){ // we start with the empty tail
    KuraShi_Tail ksty0(kurashi.dim,y0.ptail->big_m); // this defines ksty0=0
    y0.assigntail(ksty0);
  }



  if(!manyattempts){
    galsimpleintervaliteration(*PMap,*inclPMap,*inclode,xb,delta,numLohner,y0,report,
              coord,dimdiag,iternum,set_partition,testasymmetry,asymtestPMap);
  }else{
    incl_poinc_manysizes(galsimpleintervaliteration,*PMap,*inclPMap,
           *inclode,numLohner,xb,maxdelta,mindelta,y0,report,coord, dimdiag,
                        iternum,set_partition,testasymmetry,asymtestPMap);
  }

  delete asymtestPMap;
  asymtestPMap=NULL; // this should be removed from memory before PMap
                    // reason: uses section defined and destroyed in PMap

  delete inclPMap; // this should be removed from memory before PMap
                    // reason: uses section defined and destroyed in PMap
  inclPMap=NULL;
  delete inclode;
  inclode=NULL;


  delete PMap;
  PMap=NULL;
  delete [] boxes;
  boxes=NULL;
  delete [] galerr;
  galerr=NULL;

  delete ks_tail;
  ks_tail=NULL;

  cout << "the report is in [" << repname << "]" << endl;
  #if defined(WIN95) && defined(KRAK)
    if(!manyattempts)
      waitBt();
  #endif

}





void ksPDEcomputetraj(KuraShi *pkurashi)
{
  bool savetraj=true;
  char report[80];
  bool saveestm=true;
  char repestm[80];
  strcpy(report,catwork);
  strcat(report,"pdeorb.txt");

  strcpy(repestm,catwork);
  strcat(repestm,"estmpdeo.txt");

  char comment[40];
  strcpy(comment,"");

  // setting up numerical method parameters
  double tmax,bias;
  tmax=whattmax();
  bias=whatbias();
  int numLohnerSet=whichSetTypeChose();
  InclPoincareParType  param;
  param.onesteppar.method=TAYLORM;
  param.onesteppar.doC0intersection=1;
  param.onesteppar.onestepmethod=dfincl_C0simpleLo;
  param.tmax=tmax;
  param.h=pkurashi->timestep(0);

  ksInclPoincPar(param,*pkurashi);

  // the initialization of differential inclusion - PDE with the tail evolution
  //  Galerkin_Proj_KuraShi kurashiPde(pkurashi,pkurashi->dim, Ldelta_componentwiseestm);

  int Ldelta_method, Ldelta_norm;
  incl_method_chose(&Ldelta_method,&Ldelta_norm);

  Galerkin_Proj_KuraShi kurashiPde(pkurashi,pkurashi->dim, Ldelta_method, Ldelta_norm );

  // defining the initial condition:
  interval *x0,*r0;

  x0=new interval[pkurashi->dim+1];
  r0=new interval[pkurashi->dim+1];

  intervalmatrix1 *C;
  C=new intervalmatrix1[pkurashi->dim+1];

  KuramotoPoint(x0,pkurashi->dim,chooseksp(pkurashi->mu));

  identity(C,pkurashi->dim);
  double delta=0;
  for(int i=1;i<=pkurashi->dim;i++){
    r0[i]=delta*pow(0.5,(i-1))*interval(-1,1);
  }

  int big_m;
  Set *x;
  x=new Set(pkurashi->dim,numLohnerSet,x0,C,r0);
//  x->switchingOn=true; // currently not allowed

  // now we define the tail part to 'x' - to be zero
  cout << "enter 'M' the dimension of the near-tail (> " << pkurashi->dim
       << ")=" ;
  cin >> big_m ;
  if(big_m <= pkurashi->dim){
    cerr << "M too small " << endl;
     #if defined(KRAK) && defined(WIN95)
                   waitBt();
    #endif
    exit(EXIT_FAILURE);
  }
  KuraShi_Tail ksty0(pkurashi->dim,big_m); // this defines ksty0=0
  int withtail=1;
  SetWithTail y0(pkurashi->dim,withtail,&ksty0);



  ofstream dummy;

  incl_Lo_orbit(kurashiPde,param,0,*x,y0,comment,bias,savetraj,report,
           saveestm,repestm,false,dummy);
  delete x;
  delete [] C;
  delete [] r0;
  delete [] x0;

  if(savetraj){
    cout  << "the trajectory is stored in the file " ;
    cout  << report << endl;
  }
  if(saveestm){
    cout << "the estimates are stored in the file "<< repestm <<
       endl  ;
  }
  #if defined(KRAK) && defined(WIN95)
    cout << "press any key " << endl;
    waitBt();
  #endif

}
