#if !defined(_zgmatrix_h_)
#define _zgmatrix_h_
#include"defo.h"
const int DIM=2;

class matrix
{
  public:
   int d1,d2;  // the dimensions
   double c[DIM+1][DIM+1];

   //assignments - matrices
   matrix &operator= (const matrix &a);         //assign a matrix

   //constructors
   matrix();
   matrix(int dim);  // square matrix
   matrix(int dim1,int dim2);

   void clear(void);         //assigns zero to each coord
   int nonzero();

   double trace();
   double determinant();


  friend matrix operator-(const matrix &a);  // unary minus

  //arithmetic operations
  friend matrix operator+(const matrix &a1,const matrix &a2); //sum of matrices
  friend matrix operator*(const matrix &a1,const matrix &a2); //composition of matrices
  friend matrix operator-(const matrix &a1,const matrix &a2); //diference of matrices

  friend matrix operator*(double r,const matrix &a2); // multiplication by a scalar

  friend matrix inverse(const matrix &a); // computes a^{-1}, for dim=1,2


  // output
  friend ostream &operator<<(ostream &out,const matrix &a);
  #ifdef KRAK
  friend Frame &operator<<(Frame &f, const matrix &a);
  #endif




};

/// INTERVAL CLASS

class imatrix
{
  public:
   int d1,d2;  // the dimensions
   interval c[DIM+1][DIM+1];

   //assignments - matrices
   imatrix &operator= (const imatrix &a);         //assign a imatrix

   //constructors
   imatrix();
   imatrix(int dim);  // square matrix
   imatrix(int dim1,int dim2);

   imatrix(const matrix &a);


    void clear(void);         //assigns zero to each coord
    int nonzero();

  interval trace();
  interval determinant();

  imatrix &operator=(const matrix & a);

  friend imatrix operator-(const imatrix &a);  // unary minus

  //arithmetic operations
  friend imatrix operator+(const imatrix &a1,const imatrix &a2); //sum of matrices
  friend imatrix operator*(const imatrix &a1,const imatrix &a2); //composition of matrices
  friend imatrix operator-(const imatrix &a1,const imatrix &a2); //diference of matrices

  friend imatrix operator*(const interval &r,const imatrix &a2); // multiplication by a scalar

  friend imatrix inverse(const imatrix &a); // computes a^{-1}, for dim=1,2


  // output
  friend ostream &operator<<(ostream &out,const imatrix &a);
  #ifdef KRAK
  friend Frame &operator<<(Frame &f, const imatrix &a);
  #endif




};

typedef  matrix table_matrix1[MAX_DIM+1];
typedef  imatrix table_imatrix1[MAX_DIM+1];






void upperblocktrangulartoblocks(double a[][MAX_DIM+1],int pdim,
      matrix blocks[][MAX_DIM+1],int *bnum);
//  function transforms <a> - which is block 2x2 upper triangular to
//      <blocks> - which will be upper triangular is the strict sense
//   <pdim> - dimension of <a>
//    <*bnum> - number of blocks in <blocks> on output


//  Array's of matrixes

void Bmatmult(matrix a[][MAX_DIM+1], matrix b[][MAX_DIM+1],
    matrix c[][MAX_DIM+1],int bnum);
//   c=a*b - double version  ; all matrices are bnum x bnum


void Bmatmult(imatrix a[][MAX_DIM+1], imatrix b[][MAX_DIM+1],
    imatrix c[][MAX_DIM+1],int bnum);
//   c=a*b  - interval version; all matrices are bnum x bnum

int inverse_uptr(matrix b[][MAX_DIM+1],
                 matrix d[][MAX_DIM+1],int bnum);
// d=b^{-1}  where <b> is upper diagonal matrix - double version

int inverse_uptr(imatrix b[][MAX_DIM+1],imatrix d[][MAX_DIM+1],int bnum);
// d=b^{-1}  where <b> is upper diagonal matrix - interval version


// array's of matrices end


int inverse_uptr(double a[][MAX_DIM+1],double c[][MAX_DIM+1],
    int pdim);
//  c=a^{-1}

int inverse_uptr(interval a[][MAX_DIM+1],interval c[][MAX_DIM+1],
    int pdim);
//  c=a^{-1}


int diagupbltrian(double a[][MAX_DIM+1],int pdim,
   double eigenvect[][MAX_DIM+1], double eigenval[][MAX_DIM+1]);
// diagonalization of upper block triangular matrix <a> - on the diagonal
//    we allow for 2x2-cells with complex eigenvalues
//  a procedure fails when there is an repeating eigenvalue
//  on output k-th column of <eigenvect> is k-th eigenvector of <a>
//          and k-th eigenvalue is given by eigenval[k][1] in case of real eigenvalue
//          for complex eigenvalue  we have a block eigenval[k,k+1][1,2]
//                    such that eigenval[k][1]=alpha,  [k][2]=beta
//                              eigenval[k+1][1]=-beta, eigenval[k+1][2]=alpha
#endif
